@@ -19,12 +19,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: codeset.c 1252 2006-10-06 08:29:22Z mschwerin $
+ * $Id: codeset.c 2125 2007-05-22 13:30:37Z mschwerin $
  *
  */
 
 #include "config.h"
 
+#include <errno.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -41,12 +42,14 @@
 #include "logger.h"
 #include "codeset.h"
 
+
 typedef struct {
     char *language;
     char *encoding;
     char *modifier;
 } lang_locale_t;
 
+
 static lang_locale_t lang_locales[] = {
     {"af_ZA", "iso88591", NULL},
     {"ar_AE", "iso88596", NULL},
@@ -217,6 +220,7 @@ static lang_locale_t lang_locales[] = {
     {NULL, NULL, NULL}
 };
 
+
 static const lang_locale_t *
 _get_first_lang_locale (char *lcal)
 {
@@ -235,6 +239,7 @@ _get_first_lang_locale (char *lcal)
     return NULL;
 }
 
+
 char *
 get_system_encoding (void)
 {
@@ -270,7 +275,8 @@ get_system_encoding (void)
                     *mod = '\0';
 
                 codeset = ho_strdup (enc);
-            } else {
+            }
+            else {
                 const lang_locale_t *llocale = _get_first_lang_locale (lg);
 
                 if (llocale && llocale->encoding)
@@ -279,20 +285,22 @@ get_system_encoding (void)
 
             ho_free (lg);
         }
-    } else {
+    }
+    else {
         codeset = ho_strdup (codeset);
     }
 
     return codeset;
 }
 
+
 recode_t *
 recode_init (const char *src_encoding, const char *dst_encoding)
 {
 #ifdef HAVE_ICONV
     iconv_t id;
-    char *src_enc,
-     *dst_enc;
+    char *src_enc;
+    char *dst_enc;
     recode_t *xr;
 
     if (!src_encoding || !dst_encoding)
@@ -331,29 +339,34 @@ recode_init (const char *src_encoding, const char *dst_encoding)
     return NULL;
 }
 
+
 void
 recode_done (recode_t * xr)
 {
     if (xr) {
 #ifdef HAVE_ICONV
-        if (xr->id != (iconv_t) - 1)
+        if (xr->id != (iconv_t) - 1) {
             iconv_close (xr->id);
+        }
 #endif
         ho_free (xr);
     }
 }
 
+
 char *
 recode (recode_t * xr, const char *src)
 {
-    if (!src)
+    if (!src) {
         return NULL;
+    }
+
 #ifdef HAVE_ICONV
     char *inbuf;
     char *outbuf;
-    char *buffer = NULL;
     size_t inbytes;
     size_t outbytes;
+    char *buffer = NULL;
 
     if (xr && xr->id != (iconv_t) - 1) {
         inbytes = strlen (src);
@@ -363,11 +376,11 @@ recode (recode_t * xr, const char *src)
         outbuf = buffer;
 
         while (inbytes) {
-            if (iconv (xr->id, &inbuf, &inbytes, &outbuf, &outbytes) ==
-                (size_t) - 1) {
+            if (iconv (xr->id, &inbuf, &inbytes, &outbuf, &outbytes) == -1) {
                 ho_free (buffer);
                 buffer = NULL;
-                error (_("Could not perform codeset conversion (%s)!"), src);
+                error (_("Could not perform codeset conversion '%s': %s!"),
+                       src, strerror (errno));
                 break;
             }
         }
