@@ -19,41 +19,73 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: event.h 1571 2006-11-19 17:54:55Z mschwerin $
+ * $Id: event.h 2465 2007-07-07 15:37:46Z mschwerin $
  *
  */
 
 #ifndef HAVE_EVENT_H
 #define HAVE_EVENT_H
-#ifdef __cplusplus
-extern "C" {
-#endif
 
 #include <xine.h>
 
-/*
- * oxine global events and keycodes
- */
+/// These are the possible oxine event types.
 typedef enum {
-    /* xine-lib events going to the frontend */
+    /* ***********************************************
+     * xine-lib events going to the frontend
+     * *********************************************** */
+    /// Playback has finished.
     OXINE_EVENT_PLAYBACK_FINISHED = XINE_EVENT_UI_PLAYBACK_FINISHED,
     OXINE_EVENT_CHANNELS_CHANGED = XINE_EVENT_UI_CHANNELS_CHANGED,
     OXINE_EVENT_SET_TITLE = XINE_EVENT_UI_SET_TITLE,
+    /// xine-lib wants to send us a message.
     OXINE_EVENT_MESSAGE = XINE_EVENT_UI_MESSAGE,
+    /// The format of the video output has changed.
     OXINE_EVENT_FRAME_FORMAT_CHANGED = XINE_EVENT_FRAME_FORMAT_CHANGE,
-    /* xine-lib events coming from the frontend */
+    /* ***********************************************
+     * xine-lib events coming from the frontend 
+     * *********************************************** */
+    /// A mouse button was pressed.
     OXINE_EVENT_BUTTON = XINE_EVENT_INPUT_MOUSE_BUTTON,
+    /// The mouse was moved.
     OXINE_EVENT_MOTION = XINE_EVENT_INPUT_MOUSE_MOVE,
-    /* oxine events going to the frontend */
+    /* ***********************************************
+     * oxine events going to the frontend
+     * *********************************************** */
+    /// Playback was started.
     OXINE_EVENT_PLAYBACK_STARTED = 1000,
+    /// Playback was stopped by the user.
     OXINE_EVENT_PLAYBACK_STOPPED,
+    /// There bas a playback error.
     OXINE_EVENT_PLAYBACK_ERROR,
+    /// We're waiting for an alternative stream.
     OXINE_EVENT_WAITING_FOR_ALTERNATIVE,
+    /// A keypress (keyboard, LIRC) event.
     OXINE_EVENT_KEY,
-    OXINE_EVENT_OUTPUT_FORMAT_CHANGED, // the size of the output window has changed
+    /// The size of the output window has changed.
+    OXINE_EVENT_OUTPUT_FORMAT_CHANGED,
+    /// The format of the OSD has changed.
     OXINE_EVENT_OSD_FORMAT_CHANGED,
+    /// A configuration entry has changed.
+    OXINE_EVENT_CONFIG_CHANGED,
+    /// Progress has been made.
+    OXINE_EVENT_PROGRESS,
+    /// The GUI should be repainted.
+    OXINE_EVENT_GUI_REPAINT,
+    /// The GUI is to be hidden.
+    OXINE_EVENT_GUI_HIDE,
+
+    /**
+     * This event should be sent once in a while (actually at least twice
+     * every second) to give our widget toolkit a chance of updating any
+     * widget that might need a repaint.
+     */
+    OXINE_EVENT_OTK_UPDATE,
+    /// This is used in the filelist menu to update the thumbnails.
+    OXINE_EVENT_THUMBNAIL_UPDATE,
 } oxine_event_id_t;
 
+
+/// These are the possible key events.
 typedef enum {
     /* mapped xine keys */
     OXINE_KEY_MENU1 = XINE_EVENT_INPUT_MENU1,
@@ -86,6 +118,9 @@ typedef enum {
     OXINE_KEY_GREEN = XINE_EVENT_VDR_GREEN,
     OXINE_KEY_YELLOW = XINE_EVENT_VDR_YELLOW,
     OXINE_KEY_BLUE = XINE_EVENT_VDR_BLUE,
+    OXINE_KEY_CHANNELS = XINE_EVENT_VDR_CHANNELS,
+    OXINE_KEY_SCHEDULE = XINE_EVENT_VDR_SCHEDULE,
+    OXINE_KEY_RECORDINGS = XINE_EVENT_VDR_RECORDINGS,
     /* boundary */
     OXINE_KEY_NULL = 1000,
     /* stream */
@@ -93,14 +128,16 @@ typedef enum {
     OXINE_KEY_PLAY,
     OXINE_KEY_PAUSE,
     OXINE_KEY_STOP,
-    OXINE_KEY_SEEK, // go to absolute position
-    OXINE_KEY_SKIP, // skip relative to current position
+    /// Go to an absolute position in the stream.
+    OXINE_KEY_SEEK,
+    /// Skip in the stream relative to the current position.
+    OXINE_KEY_SKIP,
     OXINE_KEY_FASTFWD,
     OXINE_KEY_FASTRWD,
     OXINE_KEY_SPEED,
     /* audio settings */
     OXINE_KEY_VOLUME,
-    OXINE_KEY_VOLMUTE,
+    OXINE_KEY_VOLUME_MUTE,
     OXINE_KEY_AUDIO_CHANNEL,
     OXINE_KEY_AUDIO_OFFSET,
     /* video settings */
@@ -117,6 +154,7 @@ typedef enum {
     OXINE_KEY_SPU_CHANNEL,
     OXINE_KEY_SPU_OFFSET,
     /* OSD navigation */
+    OXINE_KEY_SHOW_OSD,
     OXINE_KEY_FIRST,
     OXINE_KEY_LAST,
     OXINE_KEY_PAGE_UP,
@@ -124,65 +162,127 @@ typedef enum {
     OXINE_KEY_BACK,
     OXINE_KEY_HOME,
     OXINE_KEY_SELECT,
-    OXINE_KEY_REMOVE,
     OXINE_KEY_PREV_WIDGET,
     OXINE_KEY_NEXT_WIDGET,
     /* menu access */
+    OXINE_KEY_MENU_HELP,
     OXINE_KEY_MENU_MAIN,
     OXINE_KEY_MENU_MUSIC,
     OXINE_KEY_MENU_VIDEO,
-    OXINE_KEY_MENU_OSD,
-    OXINE_KEY_MENU_CURRENT_TITLE,
-    OXINE_KEY_PLAY_VDR,
-    OXINE_KEY_PLAY_DVB,
-    OXINE_KEY_PLAY_V4L,
-    OXINE_KEY_HELP,
+    OXINE_KEY_MENU_IMAGE,
+    OXINE_KEY_MENU_PLAYBACK,
+    OXINE_KEY_MENU_PLAYLIST,
+    OXINE_KEY_MENU_SETTINGS,
+    OXINE_KEY_MENU_WEATHER,
+    OXINE_KEY_TELEVISION,
+    /* misc */
+    OXINE_KEY_MENU8,
+    OXINE_KEY_SAVE,
+    OXINE_KEY_INSERT,
+    OXINE_KEY_REMOVE,
+    OXINE_KEY_CUT,
+    OXINE_KEY_PASTE,
+    OXINE_KEY_COPY,
     OXINE_KEY_EJECT,
     OXINE_KEY_PLAYMODE,
     OXINE_KEY_FULLSCREEN,
     OXINE_KEY_QUIT,
     OXINE_KEY_SHUTDOWN,
-    /* direct key input (for editbox) */
+    OXINE_KEY_ROTATE_LEFT,
+    OXINE_KEY_ROTATE_RIGHT,
+    /// Do not map the input key to an event.
     OXINE_KEY_INPUT
 } oxine_key_id_t;
 
 
 typedef enum {
-    OXINE_BUTTON_NULL,
-    OXINE_BUTTON1,
-    OXINE_BUTTON2,
-    OXINE_BUTTON3,
-    OXINE_BUTTON4,
-    OXINE_BUTTON5,
+    /// No button was pressed.
+    OXINE_MOUSE_BUTTON_NULL,
+    /// Mouse button 1 was pressed.
+    OXINE_MOUSE_BUTTON_1,
+    OXINE_MOUSE_BUTTON_LEFT = OXINE_MOUSE_BUTTON_1,
+    /// Mouse button 2 was pressed.
+    OXINE_MOUSE_BUTTON_2,
+    OXINE_MOUSE_BUTTON_MIDDLE = OXINE_MOUSE_BUTTON_2,
+    /// Mouse button 3 was pressed.
+    OXINE_MOUSE_BUTTON_3,
+    OXINE_MOUSE_BUTTON_RIGHT = OXINE_MOUSE_BUTTON_3,
+    /// Mouse button 4 was pressed.
+    OXINE_MOUSE_BUTTON_4,
+    OXINE_MOUSE_SCROLLWHEEL_UP = OXINE_MOUSE_BUTTON_4,
+    /// Mouse button 5 was pressed.
+    OXINE_MOUSE_BUTTON_5,
+    OXINE_MOUSE_SCROLLWHEEL_DOWN = OXINE_MOUSE_BUTTON_5,
+    /// Mouse button 6 was pressed.
+    OXINE_MOUSE_BUTTON_6,
+    /// Mouse button 7 was pressed.
+    OXINE_MOUSE_BUTTON_7,
 } oxine_button_id_t;
 
-typedef struct {
-    oxine_event_id_t type;
-    union {
-        oxine_key_id_t key;
-        oxine_button_id_t button;
-    } source;
-    union {
-        /* Used for mouse events. */
+
+/// This represents the source of an event.
+typedef union {
+    /// The key that was pressed.
+    oxine_key_id_t key;
+    /// The mouse button that was pressed.
+    oxine_button_id_t button;
+} oxine_event_source_t;
+
+
+/// This represents the data of an event.
+typedef union {
+    /// Information about a mouse event.
+    struct {
+        /// Information about the position of the mouse pointer.
         struct {
             int x;
             int y;
         } pos;
-        /* Used for input events. */
+        /// Information about the mouse pointer.
         struct {
-            int key;
+            int num;
             int modifier;
-        } key;
-        /* Used for key events. */
-        int how;
-    } data;
+        } button;
+    } mouse;
+
+    /// Information about a keyboard event.
+    struct {
+        int key;
+        int modifier;
+    } keyboard;
+
+    struct {
+        xine_cfg_entry_t *entry;
+    } config;
+
+    struct {
+        int percent;
+        const char *description;
+    } progress;
+
+    void *user_data;
+} oxine_event_data_t;
+
+
+/// This represents an oxine event.
+typedef struct {
+    /// The event type.
+    oxine_event_id_t type;
+    /// The source of this event.
+    oxine_event_source_t source;
+    /// The data of this event.
+    oxine_event_data_t data;
+
+    /**
+     * How much are we to change the destination parameter. When, for
+     * example, changing the volume this specifies the amount of change to
+     * apply.
+     */
+    int how;
 } oxine_event_t;
 
 typedef void (*event_handler_t) (void *user_data, oxine_event_t * event);
 
 void oxine_event_handler (void *oxine_p, oxine_event_t * ev);
 
-#ifdef __cplusplus
-}
-#endif
 #endif /* HAVE_EVENT_H */
