@@ -19,12 +19,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: heap.c 1466 2006-11-06 07:26:48Z mschwerin $
+ * $Id: heap.c 2587 2007-07-23 09:36:39Z mschwerin $
  *
  */
 #include "config.h"
 
-#include <assert.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -52,13 +51,13 @@ typedef struct prefix_tag_s prefix_tag_t;
 typedef struct postfix_tag_s postfix_tag_t;
 
 struct prefix_tag_s {
-    prefix_tag_t *prev;         /* previous object in heap      */
-    prefix_tag_t *next;         /* next object in heap          */
-    postfix_tag_t *postfix;     /* ptr to postfix object        */
-    char *filename;             /* filename ptr or NULL         */
-    long line;                  /* line number or 0             */
-    size_t size;                /* size of allocated block      */
-    void *content;              /* _gen_malloc() ptr of object  */
+    prefix_tag_t *prev;                                         /* previous object in heap      */
+    prefix_tag_t *next;                                         /* next object in heap          */
+    postfix_tag_t *postfix;                                     /* ptr to postfix object        */
+    char *filename;                                             /* filename ptr or NULL         */
+    long line;                                                  /* line number or 0             */
+    size_t size;                                                /* size of allocated block      */
+    void *content;                                              /* _gen_malloc() ptr of object  */
 };
 
 struct postfix_tag_s {
@@ -83,9 +82,8 @@ _gen_malloc (char *lpFile, int nLine, size_t wSize)
     prefix_tag_t *prefix;
 
     wSize = DOALIGN (wSize);
-    prefix =
-        (prefix_tag_t *) malloc (sizeof (prefix_tag_t) + wSize +
-                                 sizeof (postfix_tag_t));
+    prefix = (prefix_tag_t *) malloc (sizeof (prefix_tag_t) + wSize +
+                                      sizeof (postfix_tag_t));
     if (prefix) {
         AddToLinkedList (prefix);
         prefix->postfix = (postfix_tag_t *) ((char *) (prefix + 1) + wSize);
@@ -95,8 +93,9 @@ _gen_malloc (char *lpFile, int nLine, size_t wSize)
         prefix->content = prefix + 1;
         prefix->size = wSize;
         memset (prefix->content, 0, wSize);
-    } else {
-        fatal ("Could not allocate memory!");
+    }
+    else {
+        _log (LEVEL_FATAL, lpFile, nLine, "Could not allocate memory!");
         abort ();
     }
 
@@ -110,10 +109,9 @@ _gen_malloc (char *lpFile, int nLine, size_t wSize)
 void *
 _gen_realloc (char *lpFile, int nLine, void *lpOld, size_t wSize)
 {
-
     void *lpNew = NULL;
 
-  /*--- Try to realloc ---*/
+    /* Try to realloc */
     if (lpOld) {
         ho_verify (lpOld);
         prefix_tag_t *prefix = (prefix_tag_t *) lpOld - 1;
@@ -122,7 +120,7 @@ _gen_realloc (char *lpFile, int nLine, void *lpOld, size_t wSize)
 
         heap_size_cur -= prefix->size;
 
-      /*--- Try to reallocate block ---*/
+        /* Try to reallocate block */
         RemoveFromLinkedList (prefix);
         memset (prefix->postfix, 0, sizeof (postfix_tag_t));
         wSize = DOALIGN (wSize);
@@ -131,7 +129,7 @@ _gen_realloc (char *lpFile, int nLine, void *lpOld, size_t wSize)
                                                 wSize +
                                                 sizeof (postfix_tag_t));
 
-      /*--- Add new (or failed old) back in ---*/
+        /* Add new (or failed old) back in */
         lpPre = (lpNewPrefix ? lpNewPrefix : prefix);
         AddToLinkedList (lpPre);
         lpPre->postfix = (postfix_tag_t *) ((char *) (lpPre + 1) + wSize);
@@ -143,20 +141,20 @@ _gen_realloc (char *lpFile, int nLine, void *lpOld, size_t wSize)
         if (heap_size_max < heap_size_cur)
             heap_size_max = heap_size_cur;
 
-      /*--- Finish ---*/
+        /* Finish */
         lpNew = (lpNewPrefix ? &lpNewPrefix[1] : NULL);
         if (!lpNew) {
-            fatal ("Could not allocate memory!");
+            _log (LEVEL_FATAL, lpFile, nLine, "Could not allocate memory!");
             abort ();
         }
     }
 
-  /*--- Else try new allocation ---*/
+    /* Else try new allocation */
     else {
         lpNew = _gen_malloc (lpFile, nLine, wSize);
     }
 
-  /*--- Return address to object ---*/
+    /* Return address to object */
     return lpNew;
 
 }
@@ -165,8 +163,9 @@ _gen_realloc (char *lpFile, int nLine, void *lpOld, size_t wSize)
 void *
 _gen_free (void *content)
 {
-    if (!content)
+    if (!content) {
         return NULL;
+    }
 
     ho_verify (content);
 
@@ -188,7 +187,11 @@ _gen_strdup (char *lpFile, int nLine, const char *lpS)
 {
     void *lpReturn = NULL;
 
-    assert (lpS);
+    if (!lpS) {
+        _log (LEVEL_FATAL, lpFile, nLine,
+              "You tried to duplicate a NULL string!");
+        abort ();
+    }
 
     size_t wSize = (size_t) (strlen (lpS) + 1);
 
@@ -218,7 +221,6 @@ heapstat (void)
 
             RenderDesc (lpCur, buffer);
 
-      /*--- print out buffer ---*/
             debug ("%s", buffer);
             total += lpCur->size;
             chunks++;
@@ -229,9 +231,9 @@ heapstat (void)
             debug ("memory usage: %li words in %li chunks", total, chunks);
     }
 
-    debug ("maximum heap size: %8li Kbyte", heap_size_max / 1024);
-    debug ("current heap size: %8li Kbyte", heap_size_cur / 1024);
-#endif
+    debug ("maximum heap size: %8li kByte", heap_size_max / 1024);
+    debug ("current heap size: %8li kByte", heap_size_cur / 1024);
+#endif /* DEBUG */
 }
 
 
@@ -239,21 +241,20 @@ void static
 AddToLinkedList (prefix_tag_t * lpAdd)
 {
 
-    /*--- Add before current head of list ---*/
+    /* Add before current head of list */
     if (heap_head) {
         lpAdd->prev = heap_head->prev;
         (lpAdd->prev)->next = lpAdd;
         lpAdd->next = heap_head;
         (lpAdd->next)->prev = lpAdd;
     }
-
-    /*--- Else first node ---*/
+    /* Else first node */
     else {
         lpAdd->prev = lpAdd;
         lpAdd->next = lpAdd;
     }
 
-    /*--- Make new item head of list ---*/
+    /* Make new item head of list */
     heap_head = lpAdd;
 }
 
@@ -261,11 +262,11 @@ static void
 RemoveFromLinkedList (prefix_tag_t * lpRemove)
 {
 
-    /*--- Remove from doubly linked list ---*/
+    /* Remove from doubly linked list */
     (lpRemove->prev)->next = lpRemove->next;
     (lpRemove->next)->prev = lpRemove->prev;
 
-    /*--- Possibly correct head pointer ---*/
+    /* Possibly correct head pointer */
     if (lpRemove == heap_head) {
         heap_head = ((lpRemove->next == lpRemove) ? NULL : lpRemove->next);
     }
@@ -274,8 +275,6 @@ RemoveFromLinkedList (prefix_tag_t * lpRemove)
 static int
 ho_is_ok (void *content)
 {
-    assert (content);
-
     return !((long) content & (ALIGNMENT - 1));
 }
 
@@ -283,22 +282,25 @@ ho_is_ok (void *content)
 static void
 ho_verify (void *content)
 {
-    assert (content);
+    if (!content) {
+        fatal ("Heap object was NULL!");
+        abort ();
+    }
 
-    if (ho_is_ok (content)) {
-        prefix_tag_t *prefix = (prefix_tag_t *) content - 1;
-        if (prefix->content == content) {
-            if (prefix->postfix->prefix != prefix) {
-                fatal ("Heap object 0x%X (%s:%d) is not valid!",
-                       prefix->filename, prefix->line, content);
-                abort ();
-            }
-        } else {
-            fatal ("Prefix was not there!");
-            abort ();
-        }
-    } else {
-        fatal ("Content %x was not correctly aligned!", content);
+    if (!ho_is_ok (content)) {
+        fatal ("Heap object 0x%08X was not correctly aligned!", content);
+        abort ();
+    }
+
+    prefix_tag_t *prefix = (prefix_tag_t *) content - 1;
+    if (prefix->content != content) {
+        fatal ("Heap object 0x%08X does not have a valid prefix!", content);
+        abort ();
+    }
+
+    if (prefix->postfix->prefix != prefix) {
+        fatal ("Heap object 0x%08X (%s:%d) is not valid!",
+               content, prefix->filename, prefix->line);
         abort ();
     }
 }
@@ -308,19 +310,20 @@ void static
 RenderDesc (prefix_tag_t * prefix, char *lpBuffer)
 {
     if (prefix->content == &prefix[1]) {
-        sprintf (lpBuffer, "%3lu words @ 0x%08lx:", 
-				 (unsigned long) prefix->size,
+        sprintf (lpBuffer, "%3lu words @ 0x%08lx:",
+                 (unsigned long) prefix->size,
                  (unsigned long) prefix->content);
         if (prefix->filename) {
             sprintf (lpBuffer + strlen (lpBuffer), "%s:%ld ",
                      prefix->filename, prefix->line);
         }
-    } else {
+    }
+    else {
         strcpy (lpBuffer, "(bad pointer given)");
     }
 }
 
-#else // USE_OXINE_HEAP_MANAGEMENT
+#else /* USE_OXINE_HEAP_MANAGEMENT */
 
 inline void *
 _gen_malloc (size_t size)
@@ -347,7 +350,7 @@ heapstat (void)
 
 }
 
-#endif // USE_OXINE_HEAP_MANAGEMENT
+#endif /* USE_OXINE_HEAP_MANAGEMENT */
 
 char *
 _gen_strdup_printf (char *lpFile, int nLine, const char *format, ...)
@@ -365,3 +368,19 @@ _gen_strdup_printf (char *lpFile, int nLine, const char *format, ...)
     return strdup (out);
 #endif
 }
+
+
+char *
+_gen_strdup_strftime (char *lpFile, int nLine, const char *format,
+                      const struct tm *tm)
+{
+    char out[4096];
+
+    strftime (out, 4096, format, tm);
+
+#ifdef USE_OXINE_HEAP_MANAGEMENT
+    return _gen_strdup (lpFile, nLine, out);
+#else
+    return strdup (out);
+#endif
+}
