@@ -19,17 +19,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: heap.h 1182 2006-09-22 07:53:33Z mschwerin $
+ * $Id: heap.h 2587 2007-07-23 09:36:39Z mschwerin $
  */
 
 #ifndef HAVE_HEAP_H
 #define HAVE_HEAP_H
-#ifdef __cplusplus
-extern "C" {
-#endif
 
 #include <string.h>
 #include <stdlib.h>
+#include <time.h>
 #include <sys/types.h>
 
 #ifdef USE_OXINE_HEAP_MANAGEMENT
@@ -44,7 +42,7 @@ void heapstat (void);
 
 #define ho_malloc(size)             (_gen_malloc(__FILE__, __LINE__, (size_t)size))
 #define ho_realloc(ptr, size)       (_gen_realloc(__FILE__, __LINE__, ptr, (size_t)size))
-#define ho_free(ptr)                (_gen_free(ptr))
+#define ho_free(ptr)                (ptr = _gen_free(ptr))
 
 #define ho_new(type)                (_gen_malloc(__FILE__, __LINE__, sizeof(type)))
 #define ho_strdup(ptr)              (_gen_strdup(__FILE__, __LINE__, ptr))
@@ -58,7 +56,7 @@ inline void heapstat (void);
 
 #define ho_malloc(size)             (_gen_malloc ((size_t)size))
 #define ho_realloc(ptr, size)       (realloc (ptr, (size_t)size))
-#define ho_free(ptr)                (_gen_free (ptr))
+#define ho_free(ptr)                (ptr = _gen_free (ptr))
 
 #define ho_new(type)                (_gen_malloc ((size_t)sizeof(type)))
 #define ho_strdup(ptr)              (strdup (ptr))
@@ -68,7 +66,7 @@ inline void heapstat (void);
 char *_gen_strdup_printf (char *, int, const char *, ...);
 #define ho_strdup_printf(...)       (_gen_strdup_printf(__FILE__, __LINE__, __VA_ARGS__))
 
-#ifdef __cplusplus
-}
-#endif
+char *_gen_strdup_strftime (char *, int, const char *, const struct tm *);
+#define ho_strdup_strftime(...)       (_gen_strdup_strftime(__FILE__, __LINE__, __VA_ARGS__))
+
 #endif /* HAVE_HEAP_H */
