@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: list.c 783 2006-02-26 13:29:59Z mschwerin $
+ * $Id: list.c 2034 2007-05-14 17:30:04Z mschwerin $
  *
  */
 #include "config.h"
@@ -283,7 +283,8 @@ l_list_concat (l_list_t * l1, l_list_t * l2)
             l2->first->prev = l1->last;
             l1->last = l2->last;
         }
-    } else {
+    }
+    else {
         l1->first = l2->first;
         l1->last = l2->last;
     }
@@ -318,17 +319,20 @@ l_list_insert (l_list_t * list, int position, void *data)
         item->next = NULL;
         list->first = item;
         list->last = item;
-    } else if (position == 0) {
+    }
+    else if (position == 0) {
         item->prev = NULL;
         item->next = list->first;
         list->first->prev = item;
         list->first = item;
-    } else if (position >= list->length) {
+    }
+    else if (position >= list->length) {
         item->prev = list->last;
         item->next = NULL;
         list->last->next = item;
         list->last = item;
-    } else {
+    }
+    else {
         int p = 0;
         l_item_t *cur = list->first;
         while (cur && p != position) {
@@ -492,7 +496,8 @@ l_list_next (l_list_t * list, void *data)
 
     if (list->current && list->current->data == data) {
         list->current = list->current->next;
-    } else {
+    }
+    else {
         list->current = list->first;
         while (list->current) {
             if (list->current && list->current->data == data) {
@@ -527,7 +532,8 @@ l_list_prev (l_list_t * list, void *data)
 
     if (list->current && list->current->data == data) {
         list->current = list->current->prev;
-    } else {
+    }
+    else {
         list->current = list->last;
         while (list->current) {
             if (list->current && list->current->data == data) {
