@@ -18,31 +18,34 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: logger.h 1237 2006-10-05 10:57:40Z mschwerin $
+ * $Id: logger.h 2338 2007-06-27 09:32:01Z mschwerin $
  *
  */
 
 #ifndef HAVE_LOGGER_H
 #define HAVE_LOGGER_H
-#ifdef __cplusplus
-extern "C" {
-#endif
 
-#define LEVEL_FATAL (0)
-#define LEVEL_ERROR (1)
-#define LEVEL_WARN  (2)
-#define LEVEL_INFO  (3)
-#define LEVEL_DEBUG (4)
+typedef enum {
+    LEVEL_FATAL = 0,
+    LEVEL_ERROR = 1,
+    LEVEL_WARN = 2,
+    LEVEL_INFO = 3,
+    LEVEL_DEBUG = 4,
+    LEVEL_TODO = 5,
+} logger_level_t;
 
 void logger_init (const char *filename);
 void logger_free (void);
 
-void _log (int level, const char *file, int line, const char *format, ...);
+void _log (logger_level_t level, const char *file, int line,
+           const char *format, ...);
 
 #ifdef DEBUG
 #define debug(...) _log(LEVEL_DEBUG, __FILE__, __LINE__, __VA_ARGS__)
+#define todo(...) _log(LEVEL_TODO, __FILE__, __LINE__, __VA_ARGS__)
 #else
 #define debug(...) ;
+#define todo(...) _log(LEVEL_TODO, __FILE__, __LINE__, __VA_ARGS__)
 #endif
 
 #define log(level,...)  _log(level, __FILE__, __LINE__, __VA_ARGS__)
@@ -51,7 +54,4 @@ void _log (int level, const char *file, int line, const char *format, ...);
 #define error(...)      _log(LEVEL_ERROR, __FILE__, __LINE__, __VA_ARGS__)
 #define fatal(...)      _log(LEVEL_FATAL, __FILE__, __LINE__, __VA_ARGS__)
 
-#ifdef __cplusplus
-}
-#endif
 #endif /* HAVE_LOGGER_H */
