@@ -19,21 +19,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: mediamarks.h 1372 2006-10-16 10:39:45Z mschwerin $
+ * $Id: mediamarks.h 2347 2007-06-27 12:12:28Z mschwerin $
  *
  */
 
 #ifndef HAVE_MEDIAMARKS_H
 #define HAVE_MEDIAMARKS_H
-#ifdef __cplusplus
-extern "C" {
-#endif
 
-char *read_entire_file (const char *mrl, int *file_size);
-int mediamarks_read (filelist_t * filelist);
-int favorites_read (filelist_t * filelist);
 
-#ifdef __cplusplus
-}
-#endif
+bool mediamarks_read (filelist_t * filelist);
+
+
+bool mediamarks_write (filelist_t *filelist);
+
+
+bool favorites_read (filelist_t * filelist);
+
+
 #endif /* HAVE_MEDIAMARKS_H */
