@@ -18,23 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: mediamarks_favorites.h 1254 2006-10-06 08:37:02Z mschwerin $
+ * $Id: mediamarks_favorites.h 2340 2007-06-27 09:47:43Z mschwerin $
  *
  */
 
-#ifndef HAVE_MOSTLISTENED_H
-#define HAVE_MOSTLISTENED_H
-#ifdef __cplusplus
-extern "C" {
-#endif
+#ifndef HAVE_MEDIAMARKS_FAVORITES
+#define HAVE_MEDIAMARKS_FAVORITES
 
 #include "filelist.h"
 
 void favorites_add_directory (filelist_t * favorites_filelist,
-                                 const char *title, const char *mrl);
-filelist_t *favorites_new (filelist_t * toplevel_filelist);
+                              const char *title, const char *mrl);
 
-#ifdef __cplusplus
-}
-#endif
-#endif /* HAVE_MOSTLISTENED_H */
+#endif /* HAVE_MEDIAMARKS_FAVORITES */
