@@ -18,42 +18,75 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: menulist.h 1406 2006-10-21 12:25:25Z mschwerin $
+ * $Id: menulist.h 2537 2007-07-17 13:45:38Z mschwerin $
  *
  */
 
 #ifndef HAVE_MENU_LIST_H
 #define HAVE_MENU_LIST_H
-#ifdef __cplusplus
-extern "C" {
-#endif
 
 #include "list.h"
 #include "otk.h"
 
+/// This represents one item in a menu.
 typedef struct menuitem_s menuitem_t;
+/// This represents a list of menuitems.
 typedef struct menulist_s menulist_t;
 
+/// This represents one item in a menu.
 struct menuitem_s {
+    /// The title of this item.
     char *title;
 
+    /// A sub menu.
     menulist_t *submenu;
-    int is_uplink;
 
+    /// Is this item a link to a parent.
+    bool is_uplink;
+
+    /// The callback of this item.
     void (*cb) (void *data);
+
+    /// The user data that is passed to the callback.
     void *cb_data;
 };
 
+
+/// This represents a list of menuitems.
 struct menulist_s {
+    /// The list.
     l_list_t *list;
 
+    /// The parent of this list.
     menulist_t *parent;
+
+    /// The top left corner of the menu.
+    int x;
+    /// The top left corner of the menu.
+    int y;
+    /// The width of the menu.
+    int w;
+    /// The height of the menu.
+    int h;
 };
 
+
+/**
+ * Creates a new menu list reading the content from a file.
+ *
+ * @param mrl                   The MRL of the file containing the content
+ *                              of the new menu.
+ * @param parent                The parent menu. This may be NULL.
+ */
 menulist_t *menulist_new (const char *mrl, menulist_t * parent);
+
+
+/**
+ * Frees a menulist.
+ *
+ * @param menulist              The menulist object.
+ */
 void menulist_free (menulist_t * menulist);
 
-#ifdef __cplusplus
-}
-#endif
+
 #endif /* HAVE_MENU_LIST_H */
