@@ -18,31 +18,36 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: odk_event.c 1574 2006-11-19 20:48:44Z mschwerin $
+ * $Id: odk_event.c 2659 2007-08-22 15:07:45Z mschwerin $
  *
  */
 
 #include "config.h"
 
 #include <assert.h>
+#include <pthread.h>
 
 #include "heap.h"
 #include "logger.h"
+#include "mutex.h"
 #include "scheduler.h"
+#include "utils.h"
 
 #include "odk.h"
+#include "odk_http.h"
 #include "odk_joystick.h"
 #include "odk_lirc.h"
 #include "odk_private.h"
 
-/* 
- * ***************************************************************************
- * Name:            event_handlers_destroy
- * Access:          private
- *
- * Description:     Frees an event handler.
- * ***************************************************************************
+/**
+ * This mutex is used to make sure the frontend only gets one event at the
+ * time.
  */
+pthread_mutex_t frontend_mutex;
+pthread_mutexattr_t frontend_mutex_attr;
+
+
+/// Frees one event handler object.
 static void
 event_handlers_destroy (void *event_handler_p)
 {
@@ -51,7 +56,8 @@ event_handlers_destroy (void *event_handler_p)
 }
 
 
-static int
+/// Returns TRUE if the event handlers must be swapped to be in order.
+static bool
 swap_handler (void *h1_p, void *h2_p)
 {
     odk_event_handler_t *h1 = (odk_event_handler_t *) h1_p;
@@ -61,24 +67,18 @@ swap_handler (void *h1_p, void *h2_p)
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            odk_register_event_handler
- * Access:          public
- *
- * Description:     Registers an event handler.
- * ***************************************************************************
- */
 void
 odk_add_event_handler (odk_t * odk, event_handler_t eh, void *data,
                        odk_event_handler_priority_t priority)
 {
     assert (eh);
 
+    mutex_lock (&frontend_mutex);
+
     odk_event_handler_t *cur = l_list_first (odk->event_handlers);
     while (cur) {
         if (cur->event_handler == eh) {
-            return;
+            goto out_unlock;
         }
         cur = l_list_next (odk->event_handlers, cur);
     }
@@ -90,42 +90,34 @@ odk_add_event_handler (odk_t * odk, event_handler_t eh, void *data,
 
     l_list_append (odk->event_handlers, handler);
     l_list_sort (odk->event_handlers, swap_handler);
+
+  out_unlock:
+    mutex_unlock (&frontend_mutex);
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            odk_remove_event_handler
- * Access:          public
- *
- * Description:     Removes an event handler.
- * ***************************************************************************
- */
 void
 odk_del_event_handler (odk_t * odk, event_handler_t eh)
 {
     assert (eh);
 
+    mutex_lock (&frontend_mutex);
+
     odk_event_handler_t *cur = l_list_first (odk->event_handlers);
     while (cur) {
         if (cur->event_handler == eh) {
             l_list_remove (odk->event_handlers, cur);
             ho_free (cur);
-            return;
+            goto out_unlock;
         }
         cur = l_list_next (odk->event_handlers, cur);
     }
+
+  out_unlock:
+    mutex_unlock (&frontend_mutex);
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            odk_get_forward_events_to_xine
- * Access:          public
- *
- * Description:     Are events supposed to be forwarded to the xine engine.
- * ***************************************************************************
- */
 bool
 odk_get_forward_events_to_xine (odk_t * odk)
 {
@@ -133,14 +125,6 @@ odk_get_forward_events_to_xine (odk_t * odk)
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            odk_set_forward_events_to_xine
- * Access:          public
- *
- * Description:     Are events supposed to be forwarded to the xine engine.
- * ***************************************************************************
- */
 void
 odk_set_forward_events_to_xine (odk_t * odk, bool forward)
 {
@@ -148,14 +132,6 @@ odk_set_forward_events_to_xine (odk_t * odk, bool forward)
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            odk_xine_event_send
- * Access:          public
- *
- * Description:     Send an event to the xine engine.
- * ***************************************************************************
- */
 void
 odk_xine_event_send (odk_t * odk, bool force, int type)
 {
@@ -166,38 +142,26 @@ odk_xine_event_send (odk_t * odk, bool force, int type)
     ev.type = type;
     ev.data = NULL;
     ev.data_length = 0;
-    xine_event_send (odk->win->stream, &ev);
+    xine_event_send (odk->main_stream, &ev);
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            odk_oxine_event_send
- * Access:          public
- *
- * Description:     Sends an event to the registered event handler.
- * ***************************************************************************
- */
 void
 odk_oxine_event_send (odk_t * odk, oxine_event_t * event)
 {
+    mutex_lock (&frontend_mutex);
+
     odk_event_handler_t *handler = l_list_first (odk->event_handlers);
-    while (handler && !((event->type == OXINE_EVENT_KEY)
-                        && (event->source.key == OXINE_KEY_NULL))) {
+    while (handler) {
         handler->event_handler (handler->event_handler_data, event);
         handler = l_list_next (odk->event_handlers, handler);
     }
+
+    mutex_unlock (&frontend_mutex);
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            scale_mouse_position
- * Access:          private
- *
- * Description:     Scales the mouse position to the current video size.
- * ***************************************************************************
- */
+/// Scales the mouse position to the current video size.
 static void
 scale_mouse_position (odk_t * odk, int *x, int *y)
 {
@@ -219,14 +183,9 @@ scale_mouse_position (odk_t * odk, int *x, int *y)
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            odk_event_handler
- * Access:          private
- *
- * Description:     Handles events coming from the current video window and
- *                  the input plugins (LIRC, joystick).
- * ***************************************************************************
+/**
+ * Handles events coming from the current video output window and the input
+ * plugins (e.g. LIRC, joystick).
  */
 static void
 odk_event_handler (void *odk_p, oxine_event_t * event)
@@ -241,8 +200,8 @@ odk_event_handler (void *odk_p, oxine_event_t * event)
     case OXINE_EVENT_MOTION:
         /* Send an event to the xine engine. */
         if (odk->forward_events_to_xine) {
-            int vx = event->data.pos.x;
-            int vy = event->data.pos.y;
+            int vx = event->data.mouse.pos.x;
+            int vy = event->data.mouse.pos.y;
             scale_mouse_position (odk, &vx, &vy);
 
             xine_input_data_t inp;
@@ -255,72 +214,80 @@ odk_event_handler (void *odk_p, oxine_event_t * event)
             xine_event.data = &inp;
             xine_event.data_length = sizeof (inp);
 
-            xine_event_send (odk->win->stream, &xine_event);
+            xine_event_send (odk->main_stream, &xine_event);
         }
 
         /* Scale x and y to video size. */
-        if (!odk->is_unscaled_osd_available) {
-            int vx = event->data.pos.x;
-            int vy = event->data.pos.y;
+        if (!odk_osd_use_unscaled_osd (odk)) {
+            int vx = event->data.mouse.pos.x;
+            int vy = event->data.mouse.pos.y;
             scale_mouse_position (odk, &vx, &vy);
 
-            event->data.pos.x = vx;
-            event->data.pos.y = vy;
+            event->data.mouse.pos.x = vx;
+            event->data.mouse.pos.y = vy;
         }
 
         /* Scale x any y to OSD size. */
         {
-            event->data.pos.x =
-                (int) ((double) (event->data.pos.x - odk->osd.x_off) /
+            event->data.mouse.pos.x =
+                (int) ((double) (event->data.mouse.pos.x - odk->osd.x_off) /
                        odk->osd.hscale);
-            event->data.pos.y =
-                (int) ((double) (event->data.pos.y - odk->osd.y_off) /
+            event->data.mouse.pos.y =
+                (int) ((double) (event->data.mouse.pos.y - odk->osd.y_off) /
                        odk->osd.vscale);
         }
 
         break;
     case OXINE_EVENT_KEY:
+        /* If we're currently playing television we remap some keys. */
+        if (odk_current_is_television (odk)) {
+            switch (event->source.key) {
+            case OXINE_KEY_MENU2:
+                event->source.key = OXINE_KEY_CHANNELS;
+                break;
+            case OXINE_KEY_MENU3:
+                event->source.key = OXINE_KEY_SCHEDULE;
+                break;
+            case OXINE_KEY_MENU4:
+                event->source.key = OXINE_KEY_RECORDINGS;
+                break;
+            default:
+                break;
+            }
+        }
+
+        /* If we're currently playing television or images we remap some
+         * more keys. */
+        if (odk_current_is_television (odk) || odk_current_is_image (odk)) {
+            switch (event->source.key) {
+            case OXINE_KEY_MENU5:
+                event->source.key = OXINE_KEY_RED;
+                break;
+            case OXINE_KEY_MENU6:
+                event->source.key = OXINE_KEY_GREEN;
+                break;
+            case OXINE_KEY_MENU7:
+                event->source.key = OXINE_KEY_YELLOW;
+                break;
+            case OXINE_KEY_MENU8:
+                event->source.key = OXINE_KEY_BLUE;
+                break;
+            default:
+                break;
+            }
+        }
+
         /* Send an event to the xine engine. */
         if (odk->forward_events_to_xine
             && (event->source.key < OXINE_KEY_NULL)) {
             xine_event_t xine_event;
 
-            xine_event.stream = odk->win->stream;
+            xine_event.stream = odk->main_stream;
             xine_event.type = event->source.key;
             xine_event.data = NULL;
             xine_event.data_length = 0;
 
-#ifdef HAVE_VDR
-            /* map vdr events */
-            switch (xine_event.type) {
-            case XINE_EVENT_INPUT_MENU2:
-                if (odk_current_is_vdr (odk))
-                    xine_event.type = XINE_EVENT_VDR_CHANNELS;
-                break;
-            case XINE_EVENT_INPUT_MENU3:
-                if (odk_current_is_vdr (odk))
-                    xine_event.type = XINE_EVENT_VDR_SCHEDULE;
-                break;
-            case XINE_EVENT_INPUT_MENU4:
-                if (odk_current_is_vdr (odk))
-                    xine_event.type = XINE_EVENT_VDR_RECORDINGS;
-                break;
-            case XINE_EVENT_INPUT_MENU5:
-                if (odk_current_is_vdr (odk))
-                    xine_event.type = XINE_EVENT_VDR_RED;
-                break;
-            case XINE_EVENT_INPUT_MENU6:
-                if (odk_current_is_vdr (odk))
-                    xine_event.type = XINE_EVENT_VDR_GREEN;
-                break;
-            case XINE_EVENT_INPUT_MENU7:
-                if (odk_current_is_vdr (odk))
-                    xine_event.type = XINE_EVENT_VDR_YELLOW;
-                break;
-            }
-#endif
-
-            xine_event_send (odk->win->stream, &xine_event);
+            xine_event_send (odk->main_stream, &xine_event);
         }
         break;
     default:
@@ -332,23 +299,27 @@ odk_event_handler (void *odk_p, oxine_event_t * event)
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            xine_event_cb
- * Access:          private
- *
- * Description:     Handles events coming from the xine engine.
- * ***************************************************************************
- */
+/// Handles events coming from the xine engine.
 static void
-xine_event_cb (void *odk_p, const xine_event_t * xine_event)
+xine_event_handler (void *p, const xine_event_t * xine_event)
 {
-    odk_t *odk = (odk_t *) odk_p;
+    odk_t *odk = (odk_t *) p;
+
+#ifdef DEBUG
+    static bool info_printed = false;
+    if (!info_printed) {
+        debug ("   xine-lib thread: 0x%X", (int) pthread_self ());
+        info_printed = true;
+    }
+#endif
 
     switch (xine_event->type) {
     case XINE_EVENT_FRAME_FORMAT_CHANGE:
-        odk_osd_adapt_size (odk, (xine_format_change_data_t *)
-                            xine_event->data);
+        {
+            xine_format_change_data_t *data =
+                (xine_format_change_data_t *) xine_event->data;
+            odk_osd_adapt_size (odk, data);
+        }
         break;
     case XINE_EVENT_UI_PLAYBACK_FINISHED:
         if (odk->current_mode == ODK_MODE_NULL)
@@ -367,26 +338,50 @@ xine_event_cb (void *odk_p, const xine_event_t * xine_event)
         }
 
         /* The main stream has ended. */
-        if (xine_event->stream == odk->win->stream) {
+        if (xine_event->stream == odk->main_stream) {
             /* If we have alternative streams waiting we play one of them. */
             if (playlist_length (odk->current_alternatives) > 0) {
-                /* FIXME: We currently clear the alternatives when we stop
-                 *        the current stream in odk_play. This means we
-                 *        only try the first alternative. */
-                playlist_t *alts = odk->current_alternatives;
-                playitem_t *alt = playlist_first (alts);
-                while (alt) {
-                    oxine_event_t ev;
-                    ev.type = OXINE_EVENT_WAITING_FOR_ALTERNATIVE;
-                    odk_oxine_event_send (odk, &ev);
-                    if (odk_play (odk, alt->mrl, NULL, odk->current_mode))
-                        break;
-                    alt = playlist_next (alts, alt);
+                bool success = false;
+                playitem_t *item = NULL;
+                playlist_t *list = odk->current_alternatives;
+                playlist_t *copy = playlist_new (NULL, NULL);
+
+                oxine_event_t ev;
+                ev.type = OXINE_EVENT_WAITING_FOR_ALTERNATIVE;
+                odk_oxine_event_send (odk, &ev);
+
+                /* We have to lock the frontend mutex here, because we don't
+                 * want another thread to enter odk_play_stream while we're
+                 * in that function. This could happen, if the user stops
+                 * the current stream. */
+                mutex_lock (&frontend_mutex);
+
+                /* We make a copy of the alternatives, because the
+                 * alternatives of the current title are deleted by
+                 * odk_play_stream. */
+                playlist_lock (odk->current_alternatives);
+                item = playlist_first (list);
+                while (item) {
+                    playlist_add (copy, item->title, item->mrl); 
+                    item = playlist_next (list, item);
                 }
+                playlist_unlock (odk->current_alternatives);
+
+                /* Now we try to play one of the alternatives from
+                 * the list we just copied. */
+                item = playlist_first (copy);
+                while (!success && item) {
+                    success = odk_play_stream (odk, item->title, item->mrl,
+                                               NULL, 0, odk->current_mode);
+                    item = playlist_next (copy, item);
+                }
+                playlist_free (copy);
+
+                mutex_unlock (&frontend_mutex);
 
                 /* If we were not able to play one of the alternative
                  * streams, we send a stop event. */
-                if (!alt) {
+                if (!success) {
                     oxine_event_t ev;
                     ev.type = OXINE_EVENT_PLAYBACK_ERROR;
                     odk_oxine_event_send (odk, &ev);
@@ -411,26 +406,16 @@ xine_event_cb (void *odk_p, const xine_event_t * xine_event)
 
         /* We use this event to find out if the meta info has changed
          * without a new track starting (e.g. web-radio). */
-        if (xine_event->stream == odk->win->stream) {
+        if (xine_event->stream == odk->main_stream) {
             char *current_title = odk_get_meta_info (odk, META_INFO_TITLE);
 
-            if (current_title && !odk->current_title) {
-                odk->current_title = current_title;
+            if (!current_title && odk->current_title) {
                 oxine_event_t ev;
                 ev.type = OXINE_EVENT_PLAYBACK_STARTED;
                 odk_oxine_event_send (odk, &ev);
             }
 
-            else if (!current_title && odk->current_title) {
-                ho_free (odk->current_title);
-                odk->current_title = NULL;
-                oxine_event_t ev;
-                ev.type = OXINE_EVENT_PLAYBACK_STARTED;
-                odk_oxine_event_send (odk, &ev);
-            }
-
-            else if (current_title
-                     && odk->current_title
+            else if (current_title && odk->current_title
                      && (strcmp (current_title, odk->current_title) != 0)) {
                 ho_free (odk->current_title);
                 odk->current_title = current_title;
@@ -445,117 +430,137 @@ xine_event_cb (void *odk_p, const xine_event_t * xine_event)
         }
         break;
     case XINE_EVENT_PROGRESS:
-#ifdef DEBUG
-        if (xine_event->stream == odk->win->stream) {
+        if (xine_event->stream == odk->main_stream) {
             xine_progress_data_t *pevent =
                 (xine_progress_data_t *) xine_event->data;
-            debug ("%s [%d%%]", pevent->description, pevent->percent);
-        }
+
+            /* Actually we should send this event to the frontend. But as
+             * these events are generated by xine-lib while still in
+             * xine_play and because the frontend mutex is locked by the
+             * event handler thread when we're in xine_play this would lead
+             * to a deadlock. */
+#if 0
+            oxine_event_t ev;
+            ev.type = OXINE_EVENT_PROGRESS;
+            ev.data.progress.percent = pevent->percent;
+            ev.data.progress.description = pevent->description;
+            odk_oxine_event_send (odk, &ev);
+#else
+            debug ("%s %3d %%", pevent->description, pevent->percent);
 #endif
+        }
         break;
     case XINE_EVENT_MRL_REFERENCE:
         /* If we receive an alternative stream, we add it to a list of
-         * alternatives. When receiving the first alternative stream, we
-         * send an event telling the frontend, that we are waiting for the
-         * stream. */
-        if (xine_event->stream == odk->win->stream) {
+         * alternatives. We have to lock the frontend mutex before inserting
+         * into the alternatives list, because we might deadlock otherwise. */
+        if (xine_event->stream == odk->main_stream) {
             xine_mrl_reference_data_t *ref =
                 (xine_mrl_reference_data_t *) xine_event->data;
 
             debug ("Got alternative MRL [%d]: '%s'",
                    ref->alternative, ref->mrl);
 
-            playlist_insert (odk->current_alternatives,
-                             ref->alternative, ref->mrl, ref->mrl);
+            mutex_lock (&frontend_mutex);
+
+            playlist_insert (odk->current_alternatives, ref->alternative,
+                             odk->current_title, ref->mrl);
+
+            mutex_unlock (&frontend_mutex);
         }
         break;
+    case XINE_EVENT_UI_MESSAGE:
+#ifdef DEBUG
+        if (xine_event->stream == odk->main_stream) {
+            xine_ui_message_data_t *msg =
+                (xine_ui_message_data_t *) xine_event->data;
+
+            debug ("Got message from xine-lib: '%s'", msg->messages);
+        }
+#endif
+        break;
     default:
         break;
     }
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            odk_listen_to
- * Access:          public
- *
- * Description:     Tells odk to listen to an input plugin.
- * ***************************************************************************
- */
 void
 odk_listen_to (odk_t * odk, const char *id)
 {
 #ifdef HAVE_LIRC
-    if (!strcmp (id, "lirc")) {
-        if (!odk->lirc)
-            odk->lirc = start_lirc (odk, odk_event_handler);
+    if (!odk->lirc && (strcmp (id, "lirc") == 0)) {
+        odk->lirc = start_lirc (odk, odk_event_handler);
     }
 #endif /* HAVE_LIRC */
 #ifdef HAVE_JOYSTICK
-    if (!strcmp (id, "joystick")) {
-        if (!odk->joystick)
-            odk->joystick = start_joystick (odk, odk_event_handler);
+    if (!odk->joystick && (strcmp (id, "joystick") == 0)) {
+        odk->joystick = start_joystick (odk, odk_event_handler);
     }
 #endif /* HAVE_JOYSTICK */
+#ifdef HAVE_HTTP
+    if (!odk->http && (strcmp (id, "http") == 0)) {
+        odk->http = start_http (odk, odk_event_handler);
+    }
+#endif /* HAVE_HTTP */
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            odk_event_init
- * Access:          private
- *
- * Description:     Initialize ODK event handling.
- * ***************************************************************************
- */
 void
 odk_event_init (odk_t * odk)
 {
 #ifdef DEBUG
     assert (odk);
     assert (odk->win);
-    assert (odk->win->stream);
+    assert (odk->main_stream);
     assert (odk->background_stream);
     assert (odk->animation_stream);
 #endif
 
+    pthread_mutexattr_init (&frontend_mutex_attr);
+    pthread_mutexattr_settype (&frontend_mutex_attr, PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init (&frontend_mutex, &frontend_mutex_attr);
+
     odk->event_handlers = l_list_new ();
-    odk->forward_events_to_xine = 0;
+    odk->forward_events_to_xine = false;
     odk->win->set_event_handler (odk->win, odk_event_handler, odk);
 
-    xine_event_queue_t *q0 = xine_event_new_queue (odk->win->stream);
+    xine_event_queue_t *q0 = xine_event_new_queue (odk->main_stream);
     xine_event_queue_t *q1 = xine_event_new_queue (odk->background_stream);
     xine_event_queue_t *q2 = xine_event_new_queue (odk->animation_stream);
 
-    xine_event_create_listener_thread (q0, xine_event_cb, odk);
-    xine_event_create_listener_thread (q1, xine_event_cb, odk);
-    xine_event_create_listener_thread (q2, xine_event_cb, odk);
+    xine_event_create_listener_thread (q0, xine_event_handler, odk);
+    xine_event_create_listener_thread (q1, xine_event_handler, odk);
+    xine_event_create_listener_thread (q2, xine_event_handler, odk);
 }
 
 
-/* 
- * ***************************************************************************
- * Name:            odk_event_free
- * Access:          private
- *
- * Description:     Free ODK event handling.
- * ***************************************************************************
- */
 void
 odk_event_free (odk_t * odk)
 {
 #ifdef HAVE_LIRC
-    if (odk->lirc)
+    if (odk->lirc) {
         stop_lirc (odk->lirc);
-    odk->lirc = NULL;
+        odk->lirc = NULL;
+    }
 #endif
 #ifdef HAVE_JOYSTICK
-    if (odk->joystick)
+    if (odk->joystick) {
         stop_joystick (odk->joystick);
-    odk->joystick = NULL;
+        odk->joystick = NULL;
+    }
 #endif
-    if (odk->event_handlers)
+#ifdef HAVE_HTTP
+    if (odk->http) {
+        stop_http (odk->http);
+        odk->http = NULL;
+    }
+#endif /* HAVE_HTTP */
+    if (odk->event_handlers) {
         l_list_free (odk->event_handlers, event_handlers_destroy);
-    odk->event_handlers = NULL;
+        odk->event_handlers = NULL;
+    }
+
+    pthread_mutex_destroy (&frontend_mutex);
+    pthread_mutexattr_destroy (&frontend_mutex_attr);
 }
