@@ -20,7 +20,7 @@
  *
  * Thanks goes to Florian Echtler for adding Xinerama support.
  *
- * $Id: odk_keymap.h 1182 2006-09-22 07:53:33Z mschwerin $
+ * $Id: odk_keymap.h 1802 2007-02-05 22:12:57Z mschwerin $
  *
  */
 
@@ -30,9 +30,12 @@
 extern "C" {
 #endif
 
+#ifdef HAVE_LIBX11
 #include <X11/Xlib.h>
+#endif
 
 #include "event.h"
+#include "odk_xk.h"
 
 int odk_keymap_kbd2action (oxine_event_t * event, KeySym kbd_key, int kbd_modifier);
 int odk_keymap_lirc2action (oxine_event_t *event, char *lirc_action);
