@@ -18,24 +18,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: odk_lirc.h 1434 2006-10-30 14:11:17Z mschwerin $
+ * $Id: odk_lirc.h 2650 2007-08-21 13:57:50Z mschwerin $
  *
  */
 
 #ifndef HAVE_ODK_LIRC_H
 #define HAVE_ODK_LIRC_H
-#ifdef __cplusplus
-extern "C" {
-#endif
+
+#ifdef HAVE_LIRC
 
 #include "odk.h"
 
-#ifdef HAVE_LIRC
 odk_input_t *start_lirc (odk_t * odk, event_handler_t event_handler);
 void stop_lirc (odk_input_t * oi);
-#endif
 
-#ifdef __cplusplus
-}
-#endif
+#endif /* HAVE_LIRC */
 #endif /* HAVE_ODK_LIRC_H */
