@@ -21,12 +21,14 @@
  *
  * Thanks goes to Florian Echtler for adding Xinerama support.
  *
- * $Id: odk_x11.c 1571 2006-11-19 17:54:55Z mschwerin $
+ * $Id: odk_x11.c 2664 2007-08-22 17:24:17Z mschwerin $
  *
  */
 
 #include "config.h"
 
+#ifdef HAVE_LIBX11
+
 #include <assert.h>
 #include <errno.h>
 #include <math.h>
@@ -52,6 +54,14 @@
 #include <X11/extensions/Xinerama.h>
 #endif
 
+/* If buttons 6 and 7 are not defined in X.h we define them here. */
+#ifndef Button6
+#define Button6     6
+#endif
+#ifndef Button7
+#define Button7     7
+#endif
+
 #include "event.h"
 #include "heap.h"
 #include "i18n.h"
@@ -60,27 +70,35 @@
 #include "odk_plugin.h"
 #include "odk_private.h"
 #include "scheduler.h"
+#include "utils.h"
 
 #define WINDOW_TITLE                PACKAGE_NAME" version "PACKAGE_VERSION
 
+#define SCREENSAVER_TIMEOUT         (1000 * 60 * 2)
+#define MOUSECURSOR_TIMEOUT         (1000 * 10)
+
 typedef struct {
     odk_window_t window_class;
     x11_visual_t visual;
 
-    /* Keep going in the event loop. */
-    int keep_going;
+    bool show_video;
+    bool keep_going;
+    bool wm_is_ewmh_compliant;
+    bool is_fullscreen;
+
+    bool with_border;
+    bool stay_on_top;
 
     /* The event handler. */
     odk_cb event_handler;
     void *event_handler_data;
 
-    /* Is the window in fullscreen mode. */
-    int is_fullscreen;
-    int xinerama_screen;
-
     /* The X display and screen number */
     Display *display;
+    Window rootwindow;
+
     int screen;
+    int xinerama_screen;
 
     /* The output window. */
     Window window;
@@ -109,25 +127,18 @@ typedef struct {
     /* Stuff needed to hide the mouse cursor. */
     Cursor no_cursor;
     Cursor on_cursor;
-    int cursor_visible;
-    int mouse_cursor_job;
+    int mousecursor_job;
 
     /* Stuff needed to disable the screensaver. */
-    int have_xtest;
+    bool have_xtest;
     int xtest_keycode;
-    int keep_screensaver;
-    int screensaver_job;
 
-    /* Window Manager stuff. */
-    int wm_is_ewmh_compliant;
+    bool keep_screensaver;
+    int screensaver_job;
 } x11_window_t;
 
 static Atom XA_WIN_LAYER = None;
-static Atom XA_WM_DELETE_WINDOW = None;
-
-static Atom XA_NET_WM_STATE = None;
-static Atom XA_NET_WM_STATE_FULLSCREEN = None;
-static Atom XA_NET_WM_STATE_STAYS_ON_TOP = None;
+static Atom XA_DEL_WINDOW = None;
 
 #define MWM_HINTS_DECORATIONS   (1L << 1)
 #define PROP_MWM_HINTS_ELEMENTS 5
@@ -153,7 +164,7 @@ x11_error_eater (Display * display, XErrorEvent * xevent)
 
 
 static void
-wm_check_EWMH_compliancy (x11_window_t * win)
+wm_check_ewmh_compliancy (x11_window_t * win)
 {
     win->wm_is_ewmh_compliant = 0;
 
@@ -174,11 +185,11 @@ wm_check_EWMH_compliancy (x11_window_t * win)
         int (*eh) (Display *, XErrorEvent *);
         eh = XSetErrorHandler (x11_error_eater);
 
-        if ((XGetWindowProperty
-             (win->display, (RootWindow (win->display, win->screen)), atom0,
-              0, 1, False, XA_WINDOW, &type_return, &format_return,
-              &nitems_return, &bytes_after_return,
-              &prop_return)) == Success) {
+        if (XGetWindowProperty (win->display, win->rootwindow, atom0,
+                                0, 1, False, XA_WINDOW, &type_return,
+                                &format_return, &nitems_return,
+                                &bytes_after_return,
+                                &prop_return) == Success) {
 
             if ((type_return != None) && (type_return == XA_WINDOW) &&
                 (format_return == 32) && (nitems_return == 1)
@@ -216,51 +227,61 @@ wm_check_EWMH_compliancy (x11_window_t * win)
         /* Unset the error eater. */
         XSetErrorHandler (eh);
     }
-
-    if (win->wm_is_ewmh_compliant) {
-        XA_NET_WM_STATE = XInternAtom (win->display, "_NET_WM_STATE", False);
-        XA_NET_WM_STATE_FULLSCREEN =
-            XInternAtom (win->display, "_NET_WM_STATE_FULLSCREEN", False);
-        XA_NET_WM_STATE_STAYS_ON_TOP =
-            XInternAtom (win->display, "_NET_WM_STATE_STAYS_ON_TOP", False);
-    }
 }
 
 
 static void
-wm_set_ewmh_state (x11_window_t * win, Atom atom, int enable)
+wm_set_state (x11_window_t * win, Atom atom, bool enable)
 {
-    if (!win->wm_is_ewmh_compliant)
-        return;
-    if (atom == None)
+    if (!win->wm_is_ewmh_compliant) {
         return;
+    }
+
+    Atom prop = XInternAtom (win->display, "_NET_WM_STATE", False);
 
     XEvent xev;
     xev.xclient.type = ClientMessage;
-    xev.xclient.message_type = XA_NET_WM_STATE;
+    xev.xclient.message_type = prop;
     xev.xclient.display = win->display;
     xev.xclient.window = win->window;
     xev.xclient.format = 32;
-    xev.xclient.data.l[0] = (enable == 1) ? 1 : 0;
+    xev.xclient.data.l[0] = enable ? 1 : 0;
     xev.xclient.data.l[1] = atom;
     xev.xclient.data.l[2] = 0l;
     xev.xclient.data.l[3] = 0l;
     xev.xclient.data.l[4] = 0l;
 
-    XSendEvent (win->display,
-                RootWindow (win->display, win->screen),
-                True, SubstructureRedirectMask, &xev);
+    XSendEvent (win->display, win->rootwindow, True,
+                SubstructureRedirectMask, &xev);
 }
 
 
 static void
-wm_set_border (x11_window_t * win, int enable)
+wm_set_above (x11_window_t * win, bool enable)
+{
+    Atom prop = XInternAtom (win->display, "_NET_WM_STATE_ABOVE", False);
+
+    wm_set_state (win, prop, enable);
+}
+
+
+static void
+wm_set_fullscreen (x11_window_t * win, bool enable)
+{
+    Atom prop = XInternAtom (win->display, "_NET_WM_STATE_FULLSCREEN", False);
+
+    wm_set_state (win, prop, enable);
+}
+
+
+static void
+wm_set_border (x11_window_t * win, bool enable)
 {
     Atom prop = XInternAtom (win->display, "_MOTIF_WM_HINTS", False);
 
     MWMHints mwmhints;
     mwmhints.flags = MWM_HINTS_DECORATIONS;
-    mwmhints.decorations = (enable == 1) ? 1 : 0;
+    mwmhints.decorations = enable;
 
     XChangeProperty (win->display, win->window, prop, prop, 32,
                      PropModeReplace, (unsigned char *) &mwmhints,
@@ -279,12 +300,19 @@ dest_size_cb (void *x11_win_p,
               double video_pixel_aspect,
               int *dest_width, int *dest_height, double *dest_pixel_aspect)
 {
-    x11_window_t *x11_win = (x11_window_t *) x11_win_p;
-    assert (x11_win);
+    x11_window_t *win = (x11_window_t *) x11_win_p;
+    assert (win);
 
-    *dest_width = x11_win->window_class.output_width;
-    *dest_height = x11_win->window_class.output_height;
-    *dest_pixel_aspect = x11_win->monitor_pixel_aspect;
+    if (win->show_video) {
+        *dest_width = win->window_width;
+        *dest_height = win->window_height;
+    }
+    else {
+        *dest_width = 1;
+        *dest_height = 1;
+    }
+
+    *dest_pixel_aspect = win->monitor_pixel_aspect;
 }
 
 
@@ -296,19 +324,25 @@ frame_output_cb (void *x11_win_p,
                  int *dest_width, int *dest_height,
                  double *dest_pixel_aspect, int *win_x, int *win_y)
 {
-    x11_window_t *x11_win = (x11_window_t *) x11_win_p;
-    assert (x11_win);
+    x11_window_t *win = (x11_window_t *) x11_win_p;
+    assert (win);
 
-    *win_x = x11_win->window_xpos;
-    *win_y = x11_win->window_ypos;
+    *win_x = win->window_xpos;
+    *win_y = win->window_ypos;
 
-    *dest_x = x11_win->window_class.output_xpos;
-    *dest_y = x11_win->window_class.output_ypos;
+    *dest_x = 0;
+    *dest_y = 0;
 
-    *dest_width = x11_win->window_class.output_width;
-    *dest_height = x11_win->window_class.output_height;
+    if (win->show_video) {
+        *dest_width = win->window_width;
+        *dest_height = win->window_height;
+    }
+    else {
+        *dest_width = 1;
+        *dest_height = 1;
+    }
 
-    *dest_pixel_aspect = x11_win->monitor_pixel_aspect;
+    *dest_pixel_aspect = win->monitor_pixel_aspect;
 }
 
 
@@ -335,6 +369,7 @@ connect_x11_server (x11_window_t * win)
 
     /* Get the default screen. */
     win->screen = DefaultScreen (win->display);
+    win->rootwindow = RootWindow (win->display, win->screen);
 
     win->display_width = DisplayWidth (win->display, win->screen);
     win->display_height = DisplayHeight (win->display, win->screen);
@@ -356,7 +391,6 @@ connect_x11_server (x11_window_t * win)
     win->monitor_pixel_aspect = res_v / res_h;
 
     /* If we're using xinerama, we assume square pixels. */
-    // FIXME: why (copied from xine-ui)
 #ifdef HAVE_XINERAMA
     int event_base;
     int error_base;
@@ -383,12 +417,11 @@ connect_x11_server (x11_window_t * win)
 
     debug ("calculated pixel aspect ratio: %.02f", win->monitor_pixel_aspect);
 
-    XA_WM_DELETE_WINDOW =
-        XInternAtom (win->display, "WM_DELETE_WINDOW", False);
+    XA_DEL_WINDOW = XInternAtom (win->display, "WM_DELETE_WINDOW", False);
     XA_WIN_LAYER = XInternAtom (win->display, "_WIN_LAYER", False);
 
     /* Check for EWMH compliancy. */
-    wm_check_EWMH_compliancy (win);
+    wm_check_ewmh_compliancy (win);
 
     XUnlockDisplay (win->display);
 
@@ -444,8 +477,9 @@ show_mouse_cursor (void *x11_win_p)
     XFlush (win->display);
     XUnlockDisplay (win->display);
 
-    cancel_job (win->mouse_cursor_job);
-    win->mouse_cursor_job = schedule_job (6000, hide_mouse_cursor, win);
+    cancel_job (win->mousecursor_job);
+    win->mousecursor_job = schedule_job (MOUSECURSOR_TIMEOUT,
+                                         hide_mouse_cursor, win);
 }
 
 
@@ -465,26 +499,30 @@ screensaver_job (void *x11_win_p)
         XTestFakeKeyEvent (win->display, win->xtest_keycode,
                            False, CurrentTime);
         XSync (win->display, False);
-    } else
+    }
+    else
 #endif
     {
         XResetScreenSaver (win->display);
     }
     XUnlockDisplay (win->display);
 
-    if (access ("/usr/bin/gnome-screensaver-command", X_OK) == 0) {
-        system ("gnome-screensaver-command --poke");
-    }
+    execute_shell ("gnome-screensaver-command --poke", 0);
 
-    win->screensaver_job = schedule_job (6000, screensaver_job, win);
+    win->screensaver_job = schedule_job (SCREENSAVER_TIMEOUT,
+                                         screensaver_job, win);
 }
 
 
-/*
- * This creates a new X window.
- * returns 1 on success, 0 on error
+/**
+ * Creates a new X window.
+ *
+ * @param x                     The initial x offset.
+ * @param y                     The initial y offset.
+ * @param w                     The initial width.
+ * @param h                     The initial height.
  */
-static int
+static bool
 create_x11_window (x11_window_t * win, int x, int y, int w, int h)
 {
     win->window_xpos = x;
@@ -492,24 +530,17 @@ create_x11_window (x11_window_t * win, int x, int y, int w, int h)
     win->window_width = w;
     win->window_height = h;
 
-    win->window_class.output_xpos = 0;
-    win->window_class.output_ypos = 0;
-    win->window_class.output_width = win->window_width;
-    win->window_class.output_height = win->window_height;
-
     XLockDisplay (win->display);
 
     /* Create the window */
     unsigned long black = BlackPixel (win->display, win->screen);
-    win->window = XCreateSimpleWindow (win->display,
-                                       RootWindow (win->display, win->screen),
-                                       win->window_xpos,
-                                       win->window_ypos,
-                                       win->window_width,
-                                       win->window_height, 0, black, black);
+    win->window = XCreateSimpleWindow (win->display, win->rootwindow,
+                                       win->window_xpos, win->window_ypos,
+                                       win->window_width, win->window_height,
+                                       0, black, black);
     if (!win->window) {
         XUnlockDisplay (win->display);
-        return 0;
+        return false;
     }
 
     /* Set window title. */
@@ -539,12 +570,11 @@ create_x11_window (x11_window_t * win, int x, int y, int w, int h)
 
     /* Set the input masks */
     XSelectInput (win->display, win->window,
-                  StructureNotifyMask |
-                  ExposureMask |
+                  StructureNotifyMask | ExposureMask |
                   ButtonPressMask | PointerMotionMask | KeyPressMask);
 
     /* Set up the window delete message */
-    XSetWMProtocols (win->display, win->window, &XA_WM_DELETE_WINDOW, 1);
+    XSetWMProtocols (win->display, win->window, &XA_DEL_WINDOW, 1);
 
     /* Create the mouse cursors (on and off) */
     static char bm_no_data[] = { 0, 0, 0, 0, 0, 0, 0, 0 };
@@ -571,9 +601,6 @@ create_x11_window (x11_window_t * win, int x, int y, int w, int h)
 }
 
 
-/*
- * This destroys the window.
- */
 static int
 destroy_x11_window (x11_window_t * win)
 {
@@ -581,11 +608,6 @@ destroy_x11_window (x11_window_t * win)
     XDestroyWindow (win->display, win->window);
     XUnlockDisplay (win->display);
 
-    win->window_class.output_xpos = 0;
-    win->window_class.output_ypos = 0;
-    win->window_class.output_width = 0;
-    win->window_class.output_height = 0;
-
     win->visual.display = NULL;
     win->visual.screen = 0;
     win->visual.dest_size_cb = NULL;
@@ -595,7 +617,117 @@ destroy_x11_window (x11_window_t * win)
     return 1;
 }
 
+
+static void
+move_resize_window (odk_window_t * this, int x, int y, int width, int height)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    if (!win->is_fullscreen) {
+        if (x > 0) {
+            win->window_xpos = x;
+        }
+        if (y > 0) {
+            win->window_ypos = y;
+        }
+        if (width > 0) {
+            win->window_width = width;
+        }
+        if (height > 0) {
+            win->window_height = height;
+        }
+
+        XLockDisplay (win->display);
+        XMoveResizeWindow (win->display, win->window,
+                           win->window_xpos, win->window_ypos,
+                           win->window_width, win->window_height);
+        XFlush (win->display);
+        XUnlockDisplay (win->display);
+
+        /* Send size change event to xine. */
+        xine_port_send_gui_data (win->window_class.video_port,
+                                 XINE_GUI_SEND_DRAWABLE_CHANGED,
+                                 (void *) win->window);
+    }
+
+    else {
+        if (x > 0) {
+            win->saved_xpos = x;
+        }
+        if (y > 0) {
+            win->saved_ypos = y;
+        }
+        if (width > 0) {
+            win->saved_width = width;
+        }
+        if (height > 0) {
+            win->saved_height = height;
+        }
+    }
+}
+
+
+static void
+move_window (odk_window_t * this, int x, int y)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    int width = win->window_width;
+    int height = win->window_height;
+
+    move_resize_window (this, x, y, width, height);
+}
+
+
+static void
+resize_window (odk_window_t * this, int width, int height)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    int x = win->window_xpos;
+    int y = win->window_ypos;
+
+    move_resize_window (this, x, y, width, height);
+}
+
+
 static int
+get_window_width (odk_window_t * this)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    return win->window_width;
+}
+
+
+static int
+get_window_height (odk_window_t * this)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    return win->window_height;
+}
+
+
+static int
+get_window_id (odk_window_t * this)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    return win->window;
+}
+
+
+static void
+show_video (odk_window_t * this, bool show)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    win->show_video = show;
+}
+
+
+static bool
 is_fullscreen (odk_window_t * this)
 {
     x11_window_t *win = (x11_window_t *) this;
@@ -603,16 +735,27 @@ is_fullscreen (odk_window_t * this)
     return win->is_fullscreen;
 }
 
-/*
- * This sets and unsets the fullscreen mode.
- */
-static int
-fullscreen (odk_window_t * this, int want_fullscreen)
+
+static bool
+is_viewable (odk_window_t * this)
 {
     x11_window_t *win = (x11_window_t *) this;
 
-    if (win->is_fullscreen == want_fullscreen)
-        return 1;
+    XWindowAttributes wa;
+    XGetWindowAttributes (win->display, win->window, &wa);
+
+    return (wa.map_state == IsViewable);
+}
+
+
+static void
+set_fullscreen (odk_window_t * this, bool want_fullscreen)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    if (win->is_fullscreen == want_fullscreen) {
+        return;
+    }
 
     XLockDisplay (win->display);
 
@@ -627,52 +770,49 @@ fullscreen (odk_window_t * this, int want_fullscreen)
 #ifdef HAVE_XINERAMA
         int event_base;
         int error_base;
-
-        int screen_num;
+        int screen_use = 0;
+        int screen_num = 0;
         XineramaScreenInfo *screeninfo;
 
         if (XineramaQueryExtension (win->display, &event_base, &error_base)
             && (screeninfo = XineramaQueryScreens (win->display, &screen_num))
             && (XineramaIsActive (win->display))) {
 
-            int use_screen = 0;
-            if (win->xinerama_screen < screen_num)
-                use_screen = win->xinerama_screen;
+            if (win->xinerama_screen < screen_num) {
+                screen_use = win->xinerama_screen;
+            }
 
-            win->window_width = screeninfo[use_screen].width;
-            win->window_height = screeninfo[use_screen].height;
-            win->window_xpos = screeninfo[use_screen].x_org;
-            win->window_ypos = screeninfo[use_screen].y_org;
-        } else
+            win->window_xpos = screeninfo[screen_use].x_org;
+            win->window_ypos = screeninfo[screen_use].y_org;
+            win->window_width = screeninfo[screen_use].width;
+            win->window_height = screeninfo[screen_use].height;
+        }
+        else
 #endif
         {
-            win->window_width = win->display_width;
-            win->window_height = win->display_height;
             win->window_xpos = 0;
             win->window_ypos = 0;
+            win->window_width = win->display_width;
+            win->window_height = win->display_height;
         }
     }
 
     else {
         /* Set size and position of the output window. */
-        win->window_width = win->saved_width;
-        win->window_height = win->saved_height;
         win->window_xpos = win->saved_xpos;
         win->window_ypos = win->saved_ypos;
+        win->window_width = win->saved_width;
+        win->window_height = win->saved_height;
     }
 
-        /* Set size and position of the output area inside the window. */
-        win->window_class.output_width = win->window_width;
-        win->window_class.output_height = win->window_height;
-        win->window_class.output_xpos = 0;
-        win->window_class.output_ypos = 0;
-
     /* Go to fullscreen the modern (e.g. metacity) way. */
-    wm_set_ewmh_state (win, XA_NET_WM_STATE_FULLSCREEN, want_fullscreen);
-    wm_set_ewmh_state (win, XA_NET_WM_STATE_STAYS_ON_TOP, want_fullscreen);
+    wm_set_fullscreen (win, want_fullscreen);
 
-    /* Set/ unset window borders. */
-    wm_set_border (win, want_fullscreen ? 0 : 1);
+    /* Set window stacking. */
+    wm_set_above (win, want_fullscreen || win->stay_on_top);
+
+    /* Set window border. */
+    wm_set_border (win, !want_fullscreen && win->with_border);
 
     /* Set the input masks */
     if (want_fullscreen) {
@@ -680,7 +820,8 @@ fullscreen (odk_window_t * this, int want_fullscreen)
                       ExposureMask | KeyPressMask | ButtonPressMask |
                       StructureNotifyMask | FocusChangeMask |
                       PointerMotionMask);
-    } else {
+    }
+    else {
         XSelectInput (win->display, win->window,
                       ExposureMask | KeyPressMask | ButtonPressMask |
                       StructureNotifyMask | PointerMotionMask);
@@ -695,7 +836,9 @@ fullscreen (odk_window_t * this, int want_fullscreen)
     XRaiseWindow (win->display, win->window);
 
     /* Set the input focus */
-    XSetInputFocus (win->display, win->window, RevertToNone, CurrentTime);
+    if (is_viewable (this)) {
+        XSetInputFocus (win->display, win->window, RevertToNone, CurrentTime);
+    }
 
     XFlush (win->display);
     XUnlockDisplay (win->display);
@@ -708,15 +851,52 @@ fullscreen (odk_window_t * this, int want_fullscreen)
     show_mouse_cursor (win);
 
     win->is_fullscreen = want_fullscreen;
-
-    return 1;
 }
 
+
+static void
+set_with_border (odk_window_t * this, bool with_border)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    win->with_border = with_border;
+
+    if (!win->is_fullscreen) {
+        XLockDisplay (win->display);
+        wm_set_border (win, with_border);
+        XFlush (win->display);
+        XUnlockDisplay (win->display);
+
+        /* Send size change event to xine. */
+        xine_port_send_gui_data (win->window_class.video_port,
+                                 XINE_GUI_SEND_DRAWABLE_CHANGED,
+                                 (void *) win->window);
+    }
+}
+
+
+static void
+set_stay_on_top (odk_window_t * this, bool stay_on_top)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    win->stay_on_top = stay_on_top;
+
+    if (!win->is_fullscreen) {
+        XLockDisplay (win->display);
+        wm_set_above (win, stay_on_top);
+        XFlush (win->display);
+        XUnlockDisplay (win->display);
+    }
+}
+
+
 static void
 handle_event (x11_window_t * win, XEvent * event)
 {
-    if (!win->event_handler)
+    if (!win->event_handler) {
         return;
+    }
 
     switch (event->type) {
     case Expose:
@@ -726,37 +906,38 @@ handle_event (x11_window_t * win, XEvent * event)
         }
         break;
     case ConfigureNotify:
-        {
-            XConfigureEvent *cev = (XConfigureEvent *) event;
+        if (event->xany.window == win->window) {
+            XConfigureEvent *cevent = (XConfigureEvent *) event;
 
-            if (win->is_fullscreen)
-                break;
+            debug ("window size has changed +%d+%d %dx%d",
+                   cevent->x, cevent->y, cevent->width, cevent->height);
 
-            if ((cev->x == 0) && (cev->y == 0)) {
+            /* We update the display width and height to keep track of changes
+             * in the resolution. Otherwise changing the resolution and going
+             * to fullscreen afterwards leads to errors. */
+            win->display_width = DisplayWidth (win->display, win->screen);
+            win->display_height = DisplayHeight (win->display, win->screen);
+
+            win->window_width = cevent->width;
+            win->window_height = cevent->height;
+
+            if ((cevent->x == 0) && (cevent->y == 0)) {
                 Window tmp_win;
-                XLockDisplay (cev->display);
-                XTranslateCoordinates (cev->display, cev->window,
-                                       RootWindow (cev->display, win->screen),
-                                       0, 0,
-                                       &win->window_xpos,
-                                       &win->window_ypos, &tmp_win);
-                XUnlockDisplay (cev->display);
-            } else {
-                win->window_xpos = cev->x;
-                win->window_ypos = cev->y;
+                XLockDisplay (cevent->display);
+                XTranslateCoordinates (cevent->display, cevent->window,
+                                       win->rootwindow, 0, 0,
+                                       &win->window_xpos, &win->window_ypos,
+                                       &tmp_win);
+                XUnlockDisplay (cevent->display);
+            }
+            else {
+                win->window_xpos = cevent->x;
+                win->window_ypos = cevent->y;
             }
 
-            if ((win->window_width != cev->width)
-                || (win->window_height != cev->height)) {
-                win->window_width = cev->width;
-                win->window_height = cev->height;
-                win->window_class.output_width = win->window_width;
-                win->window_class.output_height = win->window_height;
-
-                oxine_event_t ev;
-                ev.type = OXINE_EVENT_OUTPUT_FORMAT_CHANGED;
-                win->event_handler (win->event_handler_data, &ev);
-            }
+            oxine_event_t ev;
+            ev.type = OXINE_EVENT_OUTPUT_FORMAT_CHANGED;
+            win->event_handler (win->event_handler_data, &ev);
         }
         break;
     case ClientMessage:
@@ -764,8 +945,9 @@ handle_event (x11_window_t * win, XEvent * event)
             XClientMessageEvent *cevent = (XClientMessageEvent *) event;
 
             /* We got a window deletion message from our window manager. */
-            if (cevent->data.l[0] == XA_WM_DELETE_WINDOW)
+            if (cevent->data.l[0] == XA_DEL_WINDOW) {
                 win->keep_going = 0;
+            }
         }
         break;
     case MotionNotify:
@@ -774,13 +956,13 @@ handle_event (x11_window_t * win, XEvent * event)
 
             oxine_event_t ev;
             ev.type = OXINE_EVENT_MOTION;
-            ev.data.pos.x = mevent->x - win->window_class.output_xpos;
-            ev.data.pos.y = mevent->y - win->window_class.output_ypos;
+            ev.data.mouse.pos.x = mevent->x;
+            ev.data.mouse.pos.y = mevent->y;
 
             win->event_handler (win->event_handler_data, &ev);
-        }
 
-        show_mouse_cursor (win);
+            show_mouse_cursor (win);
+        }
         break;
     case ButtonPress:
         {
@@ -788,31 +970,40 @@ handle_event (x11_window_t * win, XEvent * event)
 
             oxine_event_t ev;
             ev.type = OXINE_EVENT_BUTTON;
-            ev.data.pos.x = bevent->x - win->window_class.output_xpos;
-            ev.data.pos.y = bevent->y - win->window_class.output_ypos;
+            ev.data.mouse.button.num = bevent->button;
+            ev.data.mouse.button.modifier = bevent->state;
+            ev.data.mouse.pos.x = bevent->x;
+            ev.data.mouse.pos.y = bevent->y;
             switch (bevent->button) {
             case Button1:
-                ev.source.button = OXINE_BUTTON1;
+                ev.source.button = OXINE_MOUSE_BUTTON_1;
                 break;
             case Button2:
+                ev.source.button = OXINE_MOUSE_BUTTON_2;
+                break;
             case Button3:
-                ev.source.key = OXINE_KEY_MENU_OSD;
-                ev.type = OXINE_EVENT_KEY;
+                ev.source.button = OXINE_MOUSE_BUTTON_3;
                 break;
             case Button4:
-                ev.source.button = OXINE_BUTTON4;
+                ev.source.button = OXINE_MOUSE_BUTTON_4;
                 break;
             case Button5:
-                ev.source.button = OXINE_BUTTON5;
+                ev.source.button = OXINE_MOUSE_BUTTON_5;
+                break;
+            case Button6:
+                ev.source.button = OXINE_MOUSE_BUTTON_6;
+                break;
+            case Button7:
+                ev.source.button = OXINE_MOUSE_BUTTON_7;
                 break;
             default:
-                ev.source.button = OXINE_BUTTON_NULL;
+                ev.source.button = OXINE_MOUSE_BUTTON_NULL;
                 break;
             }
             win->event_handler (win->event_handler_data, &ev);
-        }
 
-        show_mouse_cursor (win);
+            show_mouse_cursor (win);
+        }
         break;
     case KeyPress:
         {
@@ -831,6 +1022,7 @@ handle_event (x11_window_t * win, XEvent * event)
             }
 
             if (((kbuf[0] != 0) && (kbuf[1] == 0))
+                || (key == XK_Escape)
                 || (key == XK_Delete)
                 || (key == XK_BackSpace)
                 || (key == XK_Left)
@@ -838,8 +1030,8 @@ handle_event (x11_window_t * win, XEvent * event)
                 || (key == XK_Home)
                 || (key == XK_End)) {
                 ev.source.key = OXINE_KEY_INPUT;
-                ev.data.key.key = key;
-                ev.data.key.modifier = kevent->state;
+                ev.data.keyboard.key = key;
+                ev.data.keyboard.modifier = kevent->state;
                 win->event_handler (win->event_handler_data, &ev);
             }
         }
@@ -847,22 +1039,43 @@ handle_event (x11_window_t * win, XEvent * event)
     default:
         break;
     }
-
 }
 
 
 static void
-event_loop (odk_window_t * this)
+enter_event_loop (odk_window_t * this)
 {
     x11_window_t *win = (x11_window_t *) this;
 
-    win->keep_going = 1;
+    win->keep_going = true;
     while (win->keep_going) {
         XEvent event;
         XNextEvent (win->display, &event);
         handle_event (win, &event);
     }
-    win->keep_going = 0;
+}
+
+
+static void
+leave_event_loop (odk_window_t * this)
+{
+    x11_window_t *win = (x11_window_t *) this;
+
+    XEvent event;
+    event.type = Expose;
+    event.xexpose.display = win->display;
+    event.xexpose.window = win->window;
+    event.xexpose.x = 0;
+    event.xexpose.y = 0;
+    event.xexpose.width = 1;
+    event.xexpose.height = 1;
+
+    win->keep_going = false;
+
+    XLockDisplay (win->display);
+    XSendEvent (win->display, win->window, False, ExposureMask, &event);
+    XFlush (win->display);
+    XUnlockDisplay (win->display);
 }
 
 
@@ -875,87 +1088,48 @@ set_event_handler (odk_window_t * this, odk_cb cb, void *data)
 }
 
 
-/*
- * Makes the window visible.
- * Returns 1 on success and 0 on error.
- */
-static int
+static bool
 show_window (odk_window_t * this)
 {
     x11_window_t *win = (x11_window_t *) this;
+
     XLockDisplay (win->display);
     XClearWindow (win->display, win->window);
     XMapWindow (win->display, win->window);
     XUnlockDisplay (win->display);
-    return 1;
+
+    return true;
 }
 
 
-/*
- * This hides the current window.
- * Returns 1 on success and 0 on error.
- */
-static int
+static bool
 hide_window (odk_window_t * this)
 {
     x11_window_t *win = (x11_window_t *) this;
+
     XLockDisplay (win->display);
     XUnmapWindow (win->display, win->window);
     XUnlockDisplay (win->display);
-    return 1;
+
+    return true;
 }
 
 
-/*
- * This changes the size and place of the current window.
- * Returns 1 on success and 0 on error.
- */
-static int
-displace_window (odk_window_t * this, int x, int y, int w, int h)
-{
-    fatal ("Tell the developers to write this function!!");
-    abort ();
-    return 0;
-}
-
-
-static void
-stop_event_loop (odk_window_t * this)
+static bool
+free_window (odk_window_t * this)
 {
     x11_window_t *win = (x11_window_t *) this;
-    XEvent event;
-    event.type = Expose;
-    event.xexpose.display = win->display;
-    event.xexpose.window = win->window;
-    event.xexpose.x = 0;
-    event.xexpose.y = 0;
-    event.xexpose.width = 1;
-    event.xexpose.height = 1;
-    win->keep_going = 0;
-    XLockDisplay (win->display);
-    XSendEvent (win->display, win->window, False, ExposureMask, &event);
-    XFlush (win->display);
-    XUnlockDisplay (win->display);
-}
 
-
-/*
- * This disposes the current window.
- * Returns 1 on success and 0 on error.
- */
-static int
-dispose_window (odk_window_t * this)
-{
-    x11_window_t *win = (x11_window_t *) this;
     /* Stop the event loop. */
     if (win->keep_going) {
-        stop_event_loop (this);
+        leave_event_loop (this);
     }
-    /* We need to lock the job mutex, because we modify the screensaver and
-     * the mouse cursor jobs. */
+
+    /* We need to lock the job mutex, because we modify 
+     * the screensaver and the mouse cursor jobs. */
     lock_job_mutex ();
     cancel_job (win->screensaver_job);
-    cancel_job (win->mouse_cursor_job);
+    cancel_job (win->mousecursor_job);
     unlock_job_mutex ();
 
     /* Close the video port and destroy the window. */
@@ -966,7 +1140,7 @@ dispose_window (odk_window_t * this)
 
     ho_free (win);
 
-    return 1;
+    return true;
 }
 
 odk_window_t *x11_construct (xine_t * xine, const char *driver);
@@ -977,20 +1151,13 @@ odk_window_t *x11_construct (xine_t * xine, const char *driver);
 odk_window_t *
 x11_construct (xine_t * xine, const char *driver)
 {
-    xine_cfg_entry_t centry;
-
     /* Create new window data */
     x11_window_t *win = ho_new (x11_window_t);
 
-    /* Fetch configuration values */
-    xine_config_lookup_entry (xine, "gui.x11.window_width", &centry);
-    int window_width = centry.num_value;
-    xine_config_lookup_entry (xine, "gui.x11.window_height", &centry);
-    int window_height = centry.num_value;
-    xine_config_lookup_entry (xine, "gui.x11.monitor_pixel_aspect", &centry);
-    double monitor_pixel_aspect = (double) centry.num_value / (double) 100;
-    xine_config_lookup_entry (xine, "gui.x11.xinerama_screen", &centry);
-    win->xinerama_screen = centry.num_value;
+    double monitor_pixel_aspect = config_get_number ("gui.x11.monitor_"
+                                                     "pixel_aspect");
+    monitor_pixel_aspect /= 100;
+    win->xinerama_screen = config_get_number ("gui.x11.xinerama_screen");
 
     /* Try to connect to the X11 server. */
     if (!connect_x11_server (win)) {
@@ -1005,7 +1172,7 @@ x11_construct (xine_t * xine, const char *driver)
     }
 
     /* Create our X11 output window. */
-    create_x11_window (win, 0, 0, window_width, window_height);
+    create_x11_window (win, 0, 0, 1, 1);
     win->window_class.xine = xine;
 
     /* Create the xine video port. */
@@ -1036,23 +1203,45 @@ x11_construct (xine_t * xine, const char *driver)
         int c;
         int d;
         win->have_xtest = XTestQueryExtension (win->display, &a, &b, &c, &d);
-        if (win->have_xtest == True)
+        if (win->have_xtest == True) {
             win->xtest_keycode = XKeysymToKeycode (win->display, XK_Shift_L);
+        }
     }
 #endif
 
-    win->screensaver_job = schedule_job (6000, screensaver_job, win);
-    win->mouse_cursor_job = schedule_job (6000, hide_mouse_cursor, win);
+    win->screensaver_job = schedule_job (SCREENSAVER_TIMEOUT,
+                                         screensaver_job, win);
+    win->mousecursor_job = schedule_job (MOUSECURSOR_TIMEOUT,
+                                         hide_mouse_cursor, win);
+
+    win->show_video = true;
+    win->with_border = true;
+    win->stay_on_top = false;
 
     win->window_class.show = show_window;
     win->window_class.hide = hide_window;
-    win->window_class.displace = displace_window;
-    win->window_class.dispose = dispose_window;
-    win->window_class.fullscreen = fullscreen;
+    win->window_class.free = free_window;
+    win->window_class.show_video = show_video;
+
+    win->window_class.set_fullscreen = set_fullscreen;
     win->window_class.is_fullscreen = is_fullscreen;
-    win->window_class.event_loop = event_loop;
-    win->window_class.stop_event_loop = stop_event_loop;
+
+    win->window_class.set_stay_on_top = set_stay_on_top;
+    win->window_class.set_with_border = set_with_border;
+
+    win->window_class.enter_event_loop = enter_event_loop;
+    win->window_class.leave_event_loop = leave_event_loop;
     win->window_class.set_event_handler = set_event_handler;
 
+    win->window_class.get_width = get_window_width;
+    win->window_class.get_height = get_window_height;
+    win->window_class.get_window_id = get_window_id;
+
+    win->window_class.move = move_window;
+    win->window_class.resize = resize_window;
+    win->window_class.move_resize = move_resize_window;
+
     return (odk_window_t *) win;
 }
+
+#endif
