@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: playlist_m3u.c 1250 2006-10-06 07:52:11Z mschwerin $
+ * $Id: playlist_m3u.c 2415 2007-07-03 09:15:51Z mschwerin $
  *
  * For information on playlist formats have a look at
  * http://gonze.com/playlists/playlist-format-survey.html
@@ -26,14 +26,6 @@
  */
 #include "config.h"
 
-#include "environment.h"
-#include "filelist.h"
-#include "heap.h"
-#include "i18n.h"
-#include "logger.h"
-#include "playlist.h"
-#include "playlist_m3u.h"
-
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -42,63 +34,15 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-
-/* 
- * ***************************************************************************
- * Name:            trim_whitespace
- * Access:          public
- *
- * Description:     Trims whitespace off the start and end of the string.
- * ***************************************************************************
- */
-char *
-trim_whitespace (char *str)
-{
-    int p = strlen (str) - 1;
-    while (p >= 0
-           && (str[p] == '\n' || str[p] == '\r' || str[p] == '\t'
-               || str[p] == ' '))
-        str[p--] = 0;
-
-    p = 0;
-    while (p <= strlen (str)
-           && (str[p] == '\n' || str[p] == '\r' || str[p] == '\t'
-               || str[p] == ' ')) {
-        p++;
-    }
-
-    return (char *) (str + p);
-}
-
-
-bool
-playlist_m3u_read (filelist_t * filelist)
-{
-    int size = 0;
-    char *buf = read_entire_file (filelist->mrl, &size);
-
-    if (!buf)
-        return false;
-
-    char *line = strtok (buf, "\n");
-    while (line) {
-        line = trim_whitespace (line);
-
-        // if its not a comment or empty, add it
-        if (strlen (line) > 0 && strstr (line, "#") != line) {
-            char *mrl = line;
-            char *title = create_title (mrl);
-            filelist_add (filelist, title, mrl, FILE_TYPE_REGULAR);
-            ho_free (title);
-        }
-
-        line = strtok (NULL, "\n");
-    }
-
-    ho_free (buf);
-
-    return true;
-}
+#include "download.h"
+#include "environment.h"
+#include "filelist.h"
+#include "heap.h"
+#include "i18n.h"
+#include "logger.h"
+#include "playlist.h"
+#include "playlist_m3u.h"
+#include "utils.h"
 
 
 bool
@@ -124,28 +68,60 @@ playlist_m3u_save (playlist_t * playlist, const char *mrl)
 bool
 playlist_m3u_load (playlist_t * playlist, const char *mrl)
 {
-    int size = 0;
-    char *buf = read_entire_file (mrl, &size);
-    if (!buf)
-        return false;
-
-    char *saveptr;
-    char *line = strtok_r (buf, "\n", &saveptr);
-    while (line) {
-        line = trim_whitespace (line);
-
-        // if its not a comment or empty, add it
-        if (strlen (line) > 0 && strstr (line, "#") != line) {
-            char *mrl = line;
-            char *title = create_title (mrl);
-            playlist_add (playlist, title, mrl);
-            ho_free (title);
-        }
-
-        line = strtok_r (NULL, "\n", &saveptr);
+    char *filename = NULL;
+    if (is_downloadable (mrl)) {
+        filename = download_to_cache (mrl, NULL, true);
+    }
+    else {
+        filename = mrl_get_filename (mrl);
     }
 
-    ho_free (buf);
+    FILE *f = fopen (filename, "r");
+    if (f == NULL) {
+        error (_("Could not open '%s': %s!"), mrl, strerror (errno));
+        return false;
+    }
 
+    char *base_dir = get_dirname (mrl);
+    char line[2048];
+    while (fgets (line, 2048, f)) {
+        char *cleaned = trim_whitespace (line);
+
+        if (cleaned[0] == '#') {
+            /* If it's a comment we ignore it. */
+        }
+        else if (strlen (cleaned) == 0) {
+            /* If it's an empty line we ignore it. */
+        }
+        else {
+            char *mrl = ho_strdup (cleaned);
+            char *title = create_title (mrl);
+
+            /* If the MRL does not start with '/' and it does not contain
+             * '://' (which we use to detect URL's e.g. http://, file://)
+             * we assume that the file is relative, and prepend the
+             * basedir of the playlist file. */
+            if ((mrl[0] != '/')
+                && (strstr (mrl, "://") == NULL)) {
+                char *tmp = ho_strdup_printf ("%s/%s", base_dir, mrl);
+                if (file_exists (tmp)) {
+                    ho_free (mrl);
+                    mrl = tmp;
+                }
+                else {
+                    ho_free (tmp);
+                }
+            }
+
+            playlist_add (playlist, title, mrl);
+
+            ho_free (title);
+            ho_free (mrl);
+        }
+    }
+    ho_free (base_dir);
+    ho_free (filename);
+
+    fclose (f);
     return true;
 }
