@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: playlist_m3u.h 1250 2006-10-06 07:52:11Z mschwerin $
+ * $Id: playlist_m3u.h 1582 2006-11-25 17:46:21Z mschwerin $
  *
  */
 
@@ -28,13 +28,7 @@
 extern "C" {
 #endif
 
-#include "filelist.h"
 #include "playlist.h"
-#include "types.h"
-
-char *trim_whitespace (char *str);
-
-bool playlist_m3u_read (filelist_t * filelist);
 
 bool playlist_m3u_save (playlist_t * playlist, const char *mrl);
 
