@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: playlist_m3u.h 1182 2006-09-22 07:53:33Z mschwerin $
+ * $Id: playlist_xml.h 1582 2006-11-25 17:46:21Z mschwerin $
  *
  */
 
@@ -28,11 +28,8 @@
 extern "C" {
 #endif
 
-#include "filelist.h"
 #include "playlist.h"
 
-bool playlist_xml_read (filelist_t * filelist);
-
 bool playlist_xml_save (playlist_t *playlist, const char *mrl);
 
 bool playlist_xml_load (playlist_t *playlist, const char *mrl);
