@@ -19,7 +19,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: scheduler.c 1408 2006-10-21 14:12:32Z mschwerin $
+ * $Id: scheduler.c 2600 2007-07-25 08:16:26Z mschwerin $
  *
  */
 #include "config.h"
@@ -52,7 +52,7 @@ typedef struct {
 } ox_scheduler_t;
 
 typedef struct {
-    struct timespec ts;         /* when to do */
+    struct timespec ts;                                         /* when to do */
     void (*cb) (void *data);
     void *data;
     int id;
@@ -69,6 +69,9 @@ scheduler_thread (void *data)
     void (*cb) (void *data);
     void *cb_data;
 
+    info (_("Successfully started scheduler thread."));
+    debug ("  scheduler thread: 0x%X", (int) pthread_self ());
+
     if (!ox_scheduler) {
         pthread_exit (NULL);
         return NULL;
@@ -80,12 +83,13 @@ scheduler_thread (void *data)
         mutex_lock (&ox_scheduler->jobs_mutex);
         job = l_list_last (ox_scheduler->jobs);
 
-        if (!job) {             /* no jobs for me */
+        if (!job) {                                             /* no jobs for me */
             mutex_unlock (&ox_scheduler->jobs_mutex);
             pthread_cond_wait (&ox_scheduler->jobs_reorganize,
                                &ox_scheduler->wait_mutex);
             continue;
-        } else {
+        }
+        else {
             mutex_unlock (&ox_scheduler->jobs_mutex);
         }
 
@@ -107,8 +111,9 @@ scheduler_thread (void *data)
             ho_free (job);
             mutex_unlock (&ox_scheduler->jobs_mutex);
 
-            if (cb)
+            if (cb) {
                 cb (cb_data);
+            }
 
             mutex_unlock (&ox_scheduler->job_execution_mutex);
         }
@@ -119,6 +124,7 @@ scheduler_thread (void *data)
     return NULL;
 }
 
+
 void
 start_scheduler (void)
 {
@@ -143,23 +149,24 @@ start_scheduler (void)
         fatal (_("Could not create scheduler thread: %s!"), strerror (errno));
         abort ();
     }
-
-    info (_("Successfully started scheduler thread."));
 }
 
+
 static void
 job_free_cb (void *j1)
 {
     ho_free (j1);
 }
 
+
 void
 stop_scheduler (void)
 {
     void *ret = NULL;
 
-    if (!ox_scheduler)
+    if (!ox_scheduler) {
         return;
+    }
 
     ox_scheduler->keep_going = 0;
     pthread_cond_signal (&ox_scheduler->jobs_reorganize);
@@ -178,12 +185,14 @@ stop_scheduler (void)
     info (_("Successfully stopped scheduler thread."));
 }
 
-static int
+
+static bool
 job_swap_cb (void *j1, void *j2)
 {
     return ((job_t *) j1)->priority < ((job_t *) j2)->priority;
 }
 
+
 int
 schedule_job (int delay, void (*cb) (void *data), void *data)
 {
@@ -193,15 +202,17 @@ schedule_job (int delay, void (*cb) (void *data), void *data)
     job_t *job = ho_new (job_t);
     int msec;
 
-    if (!ox_scheduler)
+    if (!ox_scheduler) {
         return -1;
+    }
 
     gettimeofday (&tv, &tz);
 
     job->ts.tv_sec = (delay / 1000) + tv.tv_sec;
     msec = delay % 1000;
-    if ((msec + tv.tv_usec / 1000) >= 1000)
+    if ((msec + tv.tv_usec / 1000) >= 1000) {
         job->ts.tv_sec++;
+    }
     msec = (msec + tv.tv_usec / 1000) % 1000;
     job->ts.tv_nsec = msec * 1000000;
 
@@ -220,13 +231,15 @@ schedule_job (int delay, void (*cb) (void *data), void *data)
     return ox_scheduler->job_id;
 }
 
+
 void
 cancel_job (int job_id)
 {
     job_t *job;
 
-    if (!ox_scheduler)
+    if (!ox_scheduler) {
         return;
+    }
 
     mutex_lock (&ox_scheduler->jobs_mutex);
     job = l_list_first (ox_scheduler->jobs);
@@ -242,18 +255,22 @@ cancel_job (int job_id)
     pthread_cond_signal (&ox_scheduler->jobs_reorganize);
 }
 
+
 void
 lock_job_mutex (void)
 {
-    if (!ox_scheduler)
+    if (!ox_scheduler) {
         return;
+    }
     mutex_lock (&ox_scheduler->job_execution_mutex);
 }
 
+
 void
 unlock_job_mutex (void)
 {
-    if (!ox_scheduler)
+    if (!ox_scheduler) {
         return;
+    }
     mutex_unlock (&ox_scheduler->job_execution_mutex);
 }
